/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CCachedBase.cpp

  Description :	

  Last Modified $Date: 2007/01/03 05:31:14 $

  $Revision: 1.1 $

*******************************************************************************/
#include "CCachedBase.h"
#include "CFileSystem.h"

typedef struct BaseCacheFormat {
	u32 magic;
	u32 version;
} BaseCacheFormat;

//******************************************************************************
//	
//******************************************************************************
CCachedBase::CCachedBase(CString resourcePath, CString cachedPath) :
	m_resourcePath(CFileSystem::MakeFullPath(resourcePath)),
	m_cachedPath(CFileSystem::MakeFullPath(cachedPath))
{
	fetchDates();
}

//******************************************************************************
//	
//******************************************************************************
CCachedBase::~CCachedBase()
{

}

//******************************************************************************
//	
//******************************************************************************
void CCachedBase::fetchDates()
{
	if (CFileSystem::GetModificationTime(m_resourcePath, &m_resourceDate) != 0)
		memset(&m_resourceDate, 0, sizeof(ScePspDateTime));
	
	if (CFileSystem::GetModificationTime(m_cachedPath, &m_cachedDate) != 0)
		memset(&m_cachedDate, 0, sizeof(ScePspDateTime));
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedBase::readFromCache(void* dest, const CFile* p_file, u32 size)
{
	u32 bytesRead = 0, totalBytesRead = 0;
	bool toRet = false;
	while (totalBytesRead < size && !p_file->IsEOF())
	{
		if (p_file->Read((u8*)dest + totalBytesRead, size - totalBytesRead), &bytesRead)
		{
			toRet = true;
			break;
		}
		totalBytesRead += bytesRead;
	}
	
	return toRet;
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedBase::writeToCache(void* src, const CFile* p_file, u32 size)
{
	u32 bytesWritten = 0, totalBytesWritten = 0;
	while (totalBytesWritten < size)
	{
		p_file->Write((u8*)src + totalBytesWritten, size - totalBytesWritten, &bytesWritten);
		totalBytesWritten += bytesWritten;
		if (bytesWritten == 0)
			return false;
	}
	
	return true;
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedBase::isDirty()
{
	return ((CFileSystem::DateTimeCompare(&m_resourceDate, &m_cachedDate) > 0) || !isCacheValid());
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedBase::isValidDateTime(ScePspDateTime* dateTime)
{
	return !(dateTime->year == 0
		&& dateTime->month == 0
		&& dateTime->day == 0
		&& dateTime->hour == 0
		&& dateTime->minute == 0
		&& dateTime->second == 0
		&& dateTime->microsecond == 0);
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedBase::isCacheValid()
{
	return isValidDateTime(&m_cachedDate);
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedBase::isResourceValid()
{
	return isValidDateTime(&m_resourceDate);
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedBase::initFromCache()
{
	/* The derived type is responsible for opening and closing m_pFile, but this
	 * must be called before anything is read from the file. */
	BaseCacheFormat readBuf;
	if (m_pFile == NULL)
		return false;
	
	m_pFile->Read(&readBuf, sizeof(BaseCacheFormat));
	return ((readBuf.magic == getMagicNumber()) && (readBuf.version == CACHE_FILE_VERSION));
}

//******************************************************************************
//	
//******************************************************************************
bool CCachedBase::cache()
{
	/* The derived type is responsible for opening and closing m_pFile, but this
	 * must be called before anything is written to the file. */
	BaseCacheFormat writeBuf;
	if (m_pFile == NULL)
		return false;
		
	writeBuf.magic = getMagicNumber();
	writeBuf.version = CACHE_FILE_VERSION;
	return m_pFile->Write(&writeBuf, sizeof(BaseCacheFormat));
}
