/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CConfigFile.cpp

  Description :	

  Last Modified $Date: 2007/02/09 04:20:19 $

  $Revision: 1.7 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CConfigFile.h"
#include "tinyxml.h"
#include "CFrameWork.h"
#include "CFileSystem.h"
#include <stdio.h>
#include "CConnectionManager.h"
#include "CAssert.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const CString	s_szConfigFile( "Data/SETTINGS.XML" );
static const CString	DEFAULT_OPTIONS_SKIN("Default");
static const CString	DEFAULT_OPTIONS_KEYBOARD("GUI Keyboard");
static const CString	DEFAULT_OPTIONS_MESSAGESORTSTYLE("DATE_ASC");
static const bool		DEFAULT_OPTIONS_DISPLAY_BCC_AND_CC_IN_VIEW_MESSAGE = false;
static const bool		DEFAULT_ATTRIBUTE_ACCOUNT_LEAVE_MESSAGES_ON_SERVER = false;
static const bool		DEFAULT_KEYBOARD_ANALOG_NAV = false;

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************
CSettings			*CConfigFile::s_pSettings;
TiXmlDocument 		*CConfigFile::m_pDocument;

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
bool	CConfigFile::Open()
{
	char selectedAccountName[256];
	
	s_pSettings = new CSettings();
	m_pDocument = new TiXmlDocument();

	ASSERT( m_pDocument != NULL, "Failed to create the XML document" );

	if (CFileSystem::FileExists(s_szConfigFile) == false)
	{
		s_pSettings->PopulateWithDefault();
		return true;
	}
	
	s_pSettings->accountList = new CSettings::CAccountList();

	if ( m_pDocument != NULL )
	{
		if ( m_pDocument->LoadFile( s_szConfigFile ) == true )
		{
			TiXmlHandle		doc_handle( m_pDocument );

			TiXmlElement	*p_optionsSkin(doc_handle.FirstChild("settings").FirstChild("options").FirstChild("skin").Element());
			TiXmlElement	*p_optionsKeyboard(doc_handle.FirstChild("settings").FirstChild("options").FirstChild("keyboard").Element());
			TiXmlElement	*p_optionsDisplayCC(doc_handle.FirstChild("settings").FirstChild("options").FirstChild("display_bcc_and_cc_in_view_message").Element());
			TiXmlElement	*p_optionsMessageSortStyle(doc_handle.FirstChild("settings").FirstChild("options").FirstChild("message_sort_style").Element());
			TiXmlElement	*p_connectionSelected(doc_handle.FirstChild("settings").FirstChild("connection").FirstChild("selected").Element());
			TiXmlElement	*p_connectionInternetTimeout(doc_handle.FirstChild("settings").FirstChild("connection").FirstChild("internet_timeout").Element());
			TiXmlElement	*p_connectionSocketTimeout(doc_handle.FirstChild("settings").FirstChild("connection").FirstChild("socket_timeout").Element());
			
			// Parse the version
			if (doc_handle.FirstChild("settings").Element()->Attribute("version") != NULL) {
				CConfigFile::s_pSettings->version = atoi(doc_handle.FirstChild("settings").Element()->Attribute("version"));
			} else {
				CConfigFile::s_pSettings->version = 0;
			}

			// Parse the skin
			if (p_optionsSkin != NULL && p_optionsSkin->GetText() != NULL && strlen(p_optionsSkin->GetText()) > 0) {
				CConfigFile::s_pSettings->skin = CString(p_optionsSkin->GetText());
			} else {
				ASSERT(0, "Warn loading settings: Used default skin");
				CConfigFile::s_pSettings->skin = DEFAULT_OPTIONS_SKIN;
			}
			
			// Parse the keyboard
			if (p_optionsKeyboard != NULL && p_optionsKeyboard->GetText() != NULL && strlen(p_optionsKeyboard->GetText()) > 0) {
				CConfigFile::s_pSettings->keyboard = CString(p_optionsKeyboard->GetText());
				if (p_optionsKeyboard->Attribute("analogNav") != NULL && strlen(p_optionsKeyboard->Attribute("analogNav")) > 0) {
					CConfigFile::s_pSettings->analogNav = CString(p_optionsKeyboard->Attribute("analogNav")).Equals("true");
				} else {
					CConfigFile::s_pSettings->analogNav = DEFAULT_KEYBOARD_ANALOG_NAV;
				}
			} else {
				ASSERT(0, "Warn loading settings: Used default keyboard");
				CConfigFile::s_pSettings->keyboard = DEFAULT_OPTIONS_KEYBOARD;
				CConfigFile::s_pSettings->analogNav = DEFAULT_KEYBOARD_ANALOG_NAV;
			}

			// Parse cc/bcc element
			if (p_optionsDisplayCC != NULL && p_optionsDisplayCC->GetText() != NULL && strlen(p_optionsDisplayCC->GetText()) > 0) {
				CConfigFile::s_pSettings->display_bcc_and_cc_in_view_message = CString(p_optionsDisplayCC->GetText()).Equals("true") ? true : false;
			} else {
				ASSERT(0, "Warn loading settings: Used default display_bcc_and_cc_in_view_message");
				CConfigFile::s_pSettings->display_bcc_and_cc_in_view_message = DEFAULT_OPTIONS_DISPLAY_BCC_AND_CC_IN_VIEW_MESSAGE;
			}
			
			// Parse Message Sort Style element
			if (p_optionsMessageSortStyle != NULL && p_optionsMessageSortStyle->GetText() != NULL && strlen(p_optionsMessageSortStyle->GetText()) > 0) {
				CConfigFile::s_pSettings->message_sort_style = CString(p_optionsMessageSortStyle->GetText());
			} else {
				ASSERT(0, "Warn loading settings: Used default message_sort_style");
				CConfigFile::s_pSettings->message_sort_style = DEFAULT_OPTIONS_MESSAGESORTSTYLE;
			}

			// Parse selected connection element
			if (p_connectionSelected != NULL && p_connectionSelected->GetText() != NULL && strlen(p_connectionSelected->GetText()) > 0) {
				//ensure that connection matches one in list
				std::vector<struct MPSPConnection> connectionList;
				CConnectionManager::GetPSPConnections(connectionList);

				if (connectionList.size() == 0) {
					ASSERT(0, "There are no network connections present on this psp.  A connection is required to run PSPoste");
					return false;
				}
				
				std::vector<struct MPSPConnection>::iterator it = connectionList.begin();
				while (it != connectionList.end()) {
					if ((u32)atoi(p_connectionSelected->GetText()) == (*it).id) {
						CConfigFile::s_pSettings->connection_selected = atoi(p_connectionSelected->GetText());
						break;
					}
					it++;
				}
				
				if (it == connectionList.end()) {
					ASSERT(0, "Error loading settings: selected connection doesn't match list.  Used the first connection");
					CConfigFile::s_pSettings->connection_selected = connectionList[0].id;
				}
			} else {
				ASSERT(0, "Error loading settings: No selected connection specified");
				return false;
			}

			// Parse internet_timeout connection element
			if (p_connectionInternetTimeout != NULL && p_connectionInternetTimeout->GetText() != NULL && strlen(p_connectionInternetTimeout->GetText()) > 0)
			{
				CConfigFile::s_pSettings->connection_internet_timeout = atoi(p_connectionInternetTimeout->GetText());
			}
			else
			{
				ASSERT(0, "Error loading settings: No connection_internet_timeout specified");
				return false;
			}

			// Parse selected connection element
			if (p_connectionSocketTimeout != NULL && p_connectionSocketTimeout->GetText() != NULL && strlen(p_connectionSocketTimeout->GetText()) > 0)
			{
				CConfigFile::s_pSettings->connection_socket_timeout = atoi(p_connectionSocketTimeout->GetText());
			}
			else
			{
				ASSERT(0, "Error loading settings: No connection_socket_timeout specified");
				return false;
			}

			//
			//	Parse the account settings
			//
			bool foundSelected = false;

			// Get the account element
			TiXmlElement *p_accounts(doc_handle.FirstChild("settings").FirstChild("accounts").Element());
			if (p_accounts != NULL) {
				//Determine selected account
				if ( p_accounts->Attribute("selected") != NULL && strlen(p_accounts->Attribute("selected")) > 0)
				{
					strcpy(selectedAccountName, p_accounts->Attribute( "selected" ));
				}

				// Parse each account element
				TiXmlElement *p_account(doc_handle.FirstChild( "settings" ).FirstChild( "accounts" ).FirstChild("account").Element());
					
				u32 accountCount = 1;
				for ( ; p_account != NULL; p_account = p_account->NextSiblingElement() )
				{
					CAccount *p_accountClass = new CAccount();

					// Get <display_name/>
					TiXmlElement * p_displayname(p_account->FirstChildElement("display_name"));
					if (p_displayname != NULL) {
						p_accountClass->display_name = p_displayname->GetText();
					} else {
						ASSERT(0, "Error loading account: Missing display name for account");
						return false;
					}

					// Get <email_address/>
					TiXmlElement * p_emailaddress(p_account->FirstChildElement("email_address"));
					if (p_emailaddress != NULL) {
						p_accountClass->email_address = p_emailaddress->GetText();
					} else {
						ASSERT(0, "Error loading account: Missing email address for account");
						return false;
					}

	
					// Get <account_type/>
					TiXmlElement * p_accounttype(p_account->FirstChildElement("account_type"));
					if (p_accounttype != NULL) {
						p_accountClass->account_type = p_accounttype->GetText();
					} else {
						ASSERT(0, "Error loading account: Missing account type for account");
						return false;
					}

					// Get <incoming_server/>
					TiXmlElement * p_incomingserver(p_account->FirstChildElement("incoming_server"));
					if (p_incomingserver != NULL) {
						p_accountClass->incoming_server = p_incomingserver->GetText();
					} else {
						ASSERT(0, "Error loading account: Missing incoming server for account");
						return false;
					}

					
					// Get <incoming_server_port/>
					TiXmlElement * p_incomingserverport(p_account->FirstChildElement("incoming_server_port"));
					if (p_incomingserverport != NULL) {
						p_accountClass->incoming_server_port = atoi(p_incomingserverport->GetText());
					} else {
						ASSERT(0, "Error loading account: Missing incoming server port for account");
						return false;
					}

					// Get <incoming_server_username/>
					TiXmlElement * p_iusername(p_account->FirstChildElement("incoming_server_username"));
					if (p_iusername != NULL) {
						p_accountClass->incoming_server_username = p_iusername->GetText();
					} else if (CConfigFile::s_pSettings->version == 0) {
						// BEN TODO: LEGACY CODE, move settings file updating to seperate function
						p_iusername = p_account->FirstChildElement("username");
						if (p_iusername != NULL) {
							p_accountClass->incoming_server_username = p_iusername->GetText();
						} else {
							ASSERT(0, "Error loading account: Missing username in legacy settings file");
							return false;
						}
					} else {
						ASSERT(0, "Error loading account: Missing incoming username for account");
						return false;
					}

					// Get <incoming_server_password/>
					TiXmlElement * p_ipassword(p_account->FirstChildElement("incoming_server_password"));
					if (p_ipassword != NULL) {
						p_accountClass->incoming_server_password = p_ipassword->GetText();
					} else if (CConfigFile::s_pSettings->version == 0) {
						// BEN TODO: LEGACY CODE, move settings file updating to seperate function
						p_ipassword = p_account->FirstChildElement("password");
						if (p_ipassword != NULL) {
							p_accountClass->incoming_server_password = p_ipassword->GetText();
						} else {
							ASSERT(0, "Error loading account: Missing password in legacy settings file");
							return false;
						}
					} else {
						ASSERT(0, "Error loading account: Missing incoming password for account");
						return false;
					}

					// Get <incoming_server_tls/>
					TiXmlElement * p_incomingservertls(p_account->FirstChildElement("incoming_server_tls"));
					if (p_incomingservertls != NULL) {
						p_accountClass->incoming_server_tls = CString(p_incomingservertls->GetText()).Equals("true") ? true : false;
					} else if (CConfigFile::s_pSettings->version == 0) {
						// BEN TODO: LEGACY CODE, move settings file updating to seperate function
						p_accountClass->incoming_server_tls = false;
					} else {
						ASSERT(0, "Error loading account: Missing incoming server tls for account");
						return false;
					}

					// Get <outgoing_server/>
					TiXmlElement * p_outgoingserver(p_account->FirstChildElement("outgoing_server"));
					if (p_outgoingserver != NULL) {
						p_accountClass->outgoing_server = p_outgoingserver->GetText();
					} else {
						ASSERT(0, "Error loading account: Missing outgoing server for account");
						return false;
					}

					// Get <outgoing_server_port/>
					TiXmlElement * p_outgoingserverport(p_account->FirstChildElement("outgoing_server_port"));
					if (p_outgoingserverport != NULL) {
						p_accountClass->outgoing_server_port = atoi(p_outgoingserverport->GetText());
					} else {
						ASSERT(0, "Error loading account: Missing outgoing server port for account");
						return false;
					}

					// Get <outgoing_server_username/>
					TiXmlElement * p_username(p_account->FirstChildElement("outgoing_server_username"));
					if (p_username != NULL) {
						p_accountClass->outgoing_server_username = p_username->GetText();
					} else if (CConfigFile::s_pSettings->version == 0) {
						// BEN TODO: LEGACY CODE, move settings file updating to seperate function
						p_accountClass->outgoing_server_username = "";
					} else {
						ASSERT(0, "Error loading account: Missing outgoing username for account");
						return false;
					}

					// Get <outgoing_server_password/>
					TiXmlElement * p_password(p_account->FirstChildElement("outgoing_server_password"));
					if (p_password != NULL) {
						p_accountClass->outgoing_server_password = p_password->GetText();
					} else if (CConfigFile::s_pSettings->version == 0) {
						// BEN TODO: LEGACY CODE, move settings file updating to seperate function
						p_accountClass->outgoing_server_username = "";
					} else {
						ASSERT(0, "Error loading account: Missing outgoing password for account");
						return false;
					}

					// Get <outgoing_server_tls/>
					TiXmlElement * p_outgoingservertls(p_account->FirstChildElement("outgoing_server_tls"));
					if (p_outgoingservertls != NULL) {
						p_accountClass->outgoing_server_tls = CString(p_outgoingservertls->GetText()).Equals("true") ? true : false;
					} else if (CConfigFile::s_pSettings->version == 0) {
						// BEN TODO: LEGACY CODE, move settings file updating to seperate function
						p_accountClass->outgoing_server_tls = false;
					} else {
						ASSERT(0, "Error loading account: Missing outgoing server tls for account");
						return false;
					}

					// Get <leave_messages_on_server/>
					TiXmlElement * p_leavemessagesonserver(p_account->FirstChildElement("leave_messages_on_server"));
					if (p_leavemessagesonserver != NULL) {
						p_accountClass->leave_messages_on_server = CString(p_leavemessagesonserver->GetText()).Equals("true") ? true : false;
					} else {
						ASSERT(0, "Error loading account: Missing leave messages on server for account");
						return false;
					}

					// Get <reply_to/>
					TiXmlElement * p_reply_to(p_account->FirstChildElement("reply_to"));
					if (p_reply_to != NULL) {
						p_accountClass->reply_to = p_reply_to->GetText();
					} else {
						ASSERT(0, "Error loading account: Missing reply_to for account");
						return false;
					}

					// do this last
					if (strcmp(p_displayname->GetText(), selectedAccountName) == 0) {
						s_pSettings->selectedAccount = p_accountClass;
						foundSelected = true;
						CConfigFile::s_pSettings->selectedAccount = p_accountClass;
					}

					CConfigFile::s_pSettings->accountList->push_back(p_accountClass);

					accountCount++;
				}
				
				// BEN TODO: LEGACY CODE, move settings file updating to seperate function
				// In case we've been updating from a past settings file, we want to make sure the
				// new version is stored with the xml file
				CConfigFile::s_pSettings->version = LATEST_SETTINGS_VERSION;
			} else {
				ASSERT(0, "Error loading settings: Could not find account element");
				return false;
			}		
			if (!foundSelected) {
				ASSERT(0, "Error loading settings: Selected account doesn't match any of the account display names");
				return false;
			}
		}
		else
		{
			TRACE( "%s\n", m_pDocument->ErrorDesc() );
			ASSERT( 0, "Error loading settings: Failed to find file!" );
			return false;
		}
	}
	else
	{
		return false;
	}
	return true;
}

//******************************************************************************
//
//	Save the values stored in s_pSettings to the XML file
//
//******************************************************************************
void	CConfigFile::Save()
{
	//
	//	Now save all the changes to the settings file
	//
	if ( m_pDocument != NULL )
	{
		TiXmlHandle	doc_handle( m_pDocument );
		
		// Check to make sure the document isn't blank
		TiXmlElement* p_settings(doc_handle.FirstChild("settings").Element());
		if (p_settings == NULL)
		{
			// It's blank, so the file didn't exist on startup, let's build a
			// blank settings xml document
			PopulateXmlDocument(m_pDocument);
		}
		
		TiXmlElement* p_optionsSkin(doc_handle.FirstChild("settings").FirstChild("options").FirstChild("skin").Element());
		TiXmlElement* p_optionsKeyboard(doc_handle.FirstChild("settings").FirstChild("options").FirstChild("keyboard").Element());
		TiXmlElement* p_optionsDisplayCC(doc_handle.FirstChild("settings").FirstChild("options").FirstChild("display_bcc_and_cc_in_view_message").Element());
		TiXmlElement* p_optionsMessageSortStyle(doc_handle.FirstChild("settings").FirstChild("options").FirstChild("message_sort_style").Element());
		TiXmlElement* p_connectionSelected(doc_handle.FirstChild("settings").FirstChild("connection").FirstChild("selected").Element());
		TiXmlElement* p_connectionInternetTimeout(doc_handle.FirstChild("settings").FirstChild("connection").FirstChild("internet_timeout").Element());
		TiXmlElement* p_connectionSocketTimeout(doc_handle.FirstChild("settings").FirstChild("connection").FirstChild("socket_timeout").Element());
		TiXmlElement* p_accounts(doc_handle.FirstChild("settings").FirstChild("accounts").Element());

		CString			tempText("");

		// Save the settings version
		tempText.Printf("%d", CConfigFile::s_pSettings->version);
		doc_handle.FirstChild("settings").Element()->SetAttribute("version", tempText);
		
		if (p_accounts != NULL)
		{
			p_accounts->Clear();
			p_accounts->SetAttribute("selected", (const char *)s_pSettings->selectedAccount->display_name);
		
			CSettings::CAccountList::iterator it = CConfigFile::s_pSettings->accountList->begin();

			while (it != CConfigFile::s_pSettings->accountList->end())
			{
				TiXmlElement* p_account = new TiXmlElement("account");

				TiXmlElement* p_display_name = new TiXmlElement("display_name");
				p_display_name->LinkEndChild(new TiXmlText((const char *)(*it)->display_name));
				p_account->LinkEndChild(p_display_name);

				TiXmlElement* p_email_address = new TiXmlElement("email_address");
				p_email_address->LinkEndChild(new TiXmlText((const char *)(*it)->email_address));
				p_account->LinkEndChild(p_email_address);

				TiXmlElement* p_account_type = new TiXmlElement("account_type");
				p_account_type->LinkEndChild(new TiXmlText((const char *)(*it)->account_type));
				p_account->LinkEndChild(p_account_type);
				
				TiXmlElement* p_incoming_server = new TiXmlElement("incoming_server");
				p_incoming_server->LinkEndChild(new TiXmlText((const char *)(*it)->incoming_server));
				p_account->LinkEndChild(p_incoming_server);
				
				tempText.Printf("%d", (*it)->incoming_server_port);
				TiXmlElement* p_incoming_server_port = new TiXmlElement("incoming_server_port");
				p_incoming_server_port->LinkEndChild(new TiXmlText((const char *)tempText));
				p_account->LinkEndChild(p_incoming_server_port);
				
				TiXmlElement* p_iusername = new TiXmlElement("incoming_server_username");
				p_iusername->LinkEndChild(new TiXmlText((const char *)(*it)->incoming_server_username));
				p_account->LinkEndChild(p_iusername);	

				TiXmlElement* p_ipassword = new TiXmlElement("incoming_server_password");
				p_ipassword->LinkEndChild(new TiXmlText((const char *)(*it)->incoming_server_password));
				p_account->LinkEndChild(p_ipassword);
				
				TiXmlElement* p_outgoing_server = new TiXmlElement("outgoing_server");
				p_outgoing_server->LinkEndChild(new TiXmlText((const char *)(*it)->outgoing_server));
				p_account->LinkEndChild(p_outgoing_server);
				
				tempText.Printf("%d", (*it)->outgoing_server_port);
				TiXmlElement* p_outgoing_server_port = new TiXmlElement("outgoing_server_port");
				p_outgoing_server_port->LinkEndChild(new TiXmlText((const char *)tempText));
				p_account->LinkEndChild(p_outgoing_server_port);
				
				TiXmlElement* p_username = new TiXmlElement("outgoing_server_username");
				p_username->LinkEndChild(new TiXmlText((const char *)(*it)->outgoing_server_username));
				p_account->LinkEndChild(p_username);	

				TiXmlElement* p_password = new TiXmlElement("outgoing_server_password");
				p_password->LinkEndChild(new TiXmlText((const char *)(*it)->outgoing_server_password));
				p_account->LinkEndChild(p_password);
						
				TiXmlElement* p_incoming_server_tls = new TiXmlElement("incoming_server_tls");
				if ((*it)->incoming_server_tls) {
					p_incoming_server_tls->LinkEndChild(new TiXmlText("true"));
				} else {
					p_incoming_server_tls->LinkEndChild(new TiXmlText("false"));
				}
				p_account->LinkEndChild(p_incoming_server_tls);
				
				TiXmlElement* p_outgoing_server_tls = new TiXmlElement("outgoing_server_tls");
				if ((*it)->outgoing_server_tls) {
					p_outgoing_server_tls->LinkEndChild(new TiXmlText("true"));
				} else {
					p_outgoing_server_tls->LinkEndChild(new TiXmlText("false"));
				}
				p_account->LinkEndChild(p_outgoing_server_tls);
								
				TiXmlElement* p_leave_messages_on_server = new TiXmlElement("leave_messages_on_server");
				if ((*it)->leave_messages_on_server) {
					p_leave_messages_on_server->LinkEndChild(new TiXmlText("true"));
				} else {
					p_leave_messages_on_server->LinkEndChild(new TiXmlText("false"));
				}
				p_account->LinkEndChild(p_leave_messages_on_server);
								
				TiXmlElement* p_reply_to = new TiXmlElement("reply_to");
				p_reply_to->LinkEndChild(new TiXmlText((const char *)(*it)->reply_to));
				p_account->LinkEndChild(p_reply_to);

				p_accounts->LinkEndChild(p_account);

				it++;
			}		
		}

		if (p_optionsSkin != NULL)
		{
			p_optionsSkin->Clear();
			p_optionsSkin->LinkEndChild(new TiXmlText((const char *)s_pSettings->skin));
		}
		
		if (p_optionsKeyboard != NULL)
		{
			p_optionsKeyboard->Clear();
			p_optionsKeyboard->SetAttribute("analogNav", s_pSettings->analogNav ? "true" : "false");
			p_optionsKeyboard->LinkEndChild(new TiXmlText((const char *)s_pSettings->keyboard));
		}

		if (p_optionsDisplayCC != NULL)
		{
			p_optionsDisplayCC->Clear();
			if (s_pSettings->display_bcc_and_cc_in_view_message) {
				p_optionsDisplayCC->LinkEndChild(new TiXmlText("true"));
			} else {
				p_optionsDisplayCC->LinkEndChild(new TiXmlText("false"));
			}
		}
		
		if (p_optionsMessageSortStyle != NULL)
		{
			p_optionsMessageSortStyle->Clear();
			p_optionsMessageSortStyle->LinkEndChild(new TiXmlText((const char *)s_pSettings->message_sort_style));
		}

		if (p_connectionSelected != NULL)
		{
			tempText.Printf("%d", s_pSettings->connection_selected);
			p_connectionSelected->Clear();
			p_connectionSelected->LinkEndChild(new TiXmlText((const char *)tempText));
		}

		if (p_connectionInternetTimeout != NULL)
		{
			tempText.Printf("%d", s_pSettings->connection_internet_timeout);
			p_connectionInternetTimeout->Clear();
			p_connectionInternetTimeout->LinkEndChild(new TiXmlText((const char *)tempText));
		}

		if (p_connectionSocketTimeout != NULL)
		{
			tempText.Printf("%d", s_pSettings->connection_socket_timeout);
			p_connectionSocketTimeout->Clear();
			p_connectionSocketTimeout->LinkEndChild(new TiXmlText((const char *)tempText));
		}
			
		if ( (CFileSystem::FileExists( s_szConfigFile ) == false) || (CFileSystem::MakeWritable( s_szConfigFile ) == true) )
		{
			if ( m_pDocument->SaveFile( s_szConfigFile ) == false )
			{
				ASSERT( 0, "Failed to save config file!" );
			}
		}
	}
}

//******************************************************************************
//	
//******************************************************************************
void	CConfigFile::Close()
{
	CConfigFile::Save();
	SAFE_DELETE(s_pSettings);
}

//******************************************************************************
//	
//******************************************************************************
void	CConfigFile::PopulateXmlDocument(TiXmlDocument* document)
{
	TiXmlElement* p_settingsElement = new TiXmlElement("settings");
	TiXmlElement* p_optionsElement = new TiXmlElement("options");
	TiXmlElement* p_skinElement = new TiXmlElement("skin");
	TiXmlElement* p_keyboardElement = new TiXmlElement("keyboard");
	TiXmlElement* p_display_bcc_and_cc_in_view_messageElement = new TiXmlElement("display_bcc_and_cc_in_view_message");
	TiXmlElement* p_message_sort_styleElement = new TiXmlElement("message_sort_style");
	TiXmlElement* p_connectionElement = new TiXmlElement("connection");
	TiXmlElement* p_selectedElement = new TiXmlElement("selected");
	TiXmlElement* p_internet_timeoutElement = new TiXmlElement("internet_timeout");
	TiXmlElement* p_socket_timeoutElement = new TiXmlElement("socket_timeout");
	TiXmlElement* p_accountsElement = new TiXmlElement("accounts");
	document->LinkEndChild(p_settingsElement);
	p_settingsElement->LinkEndChild(p_optionsElement);
	p_optionsElement->LinkEndChild(p_skinElement);
	p_optionsElement->LinkEndChild(p_keyboardElement);
	p_optionsElement->LinkEndChild(p_display_bcc_and_cc_in_view_messageElement);
	p_optionsElement->LinkEndChild(p_message_sort_styleElement);
	p_settingsElement->LinkEndChild(p_connectionElement);
	p_connectionElement->LinkEndChild(p_selectedElement);
	p_connectionElement->LinkEndChild(p_internet_timeoutElement);
	p_connectionElement->LinkEndChild(p_socket_timeoutElement);
	p_settingsElement->LinkEndChild(p_accountsElement);
}

//******************************************************************************
//	
//******************************************************************************
CSettings::CSettings(CSettings* settings) : selectedAccount(NULL),
			accountList(NULL)
{
	skin = settings->skin;
	keyboard = settings->keyboard;
	display_bcc_and_cc_in_view_message = settings->display_bcc_and_cc_in_view_message;
	message_sort_style = settings->message_sort_style;
	version = settings->version;
	connection_selected = settings->connection_selected;
	connection_internet_timeout = settings->connection_internet_timeout;
	connection_socket_timeout = settings->connection_socket_timeout;
	analogNav = settings->analogNav;

	accountList = new CAccountList();

	CAccountList::iterator it = settings->accountList->begin();

	while(it != settings->accountList->end()) {
		if (*it == settings->selectedAccount) {
			CAccount* account = new CAccount(*it);
			accountList->push_back(account);
			selectedAccount = account;
		} else {
			accountList->push_back(new CAccount(*it));
		}
		it++;
	}
}

//******************************************************************************
//	
//******************************************************************************
CSettings::~CSettings()
{
	CAccountList::iterator it(accountList->begin());
	while (it != accountList->end())
	{
		SAFE_DELETE(*it);
		it++;
	}
	
	SAFE_DELETE(accountList);
}

//******************************************************************************
//	
//******************************************************************************
void CSettings::PopulateWithDefault()
{
	skin = "Default";
	keyboard = "Danzeff";
	display_bcc_and_cc_in_view_message = true;
	message_sort_style = "DATE_ASC";
	version = LATEST_SETTINGS_VERSION;
	connection_selected = 1;
	connection_internet_timeout = 30;
	connection_socket_timeout = 20;
	analogNav = false;
	
	accountList = new CSettings::CAccountList();
	selectedAccount = new CAccount();
	selectedAccount->display_name = "Default Account";
	selectedAccount->email_address = "you@server";
	selectedAccount->account_type = "pop";
	selectedAccount->incoming_server = "server";
	selectedAccount->incoming_server_port = 110;
	selectedAccount->incoming_server_username = "you";
	selectedAccount->incoming_server_password = "";
	selectedAccount->incoming_server_tls = false;
	selectedAccount->outgoing_server = "server";
	selectedAccount->outgoing_server_port = 25;
	selectedAccount->outgoing_server_username = "you";
	selectedAccount->outgoing_server_password = "";
	selectedAccount->outgoing_server_tls = false;
	selectedAccount->leave_messages_on_server = true;
	selectedAccount->reply_to = "";
	accountList->push_back(selectedAccount);
}

//*******************************  END OF FILE  ********************************
