/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CContacts.cpp

  Description :	

  Last Modified $Date: 2007/01/27 19:45:12 $

  $Revision: 1.3 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CContacts.h"
#include "tinyxml.h"
#include "CFileSystem.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
CString 	s_szContactsFile ( "Data/CONTACTS.XML" );

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************
CContacts::CContactList *	CContacts::s_pContacts(NULL);
TiXmlDocument *				CContacts::m_pDocument;
bool						CContacts::m_createdDefault(false);

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
bool	CContacts::Open()
{

	m_pDocument = new TiXmlDocument();

	if (CFileSystem::FileExists(s_szContactsFile) == false)
	{
		if (!m_createdDefault)
		{
			/* Ensure the default contact doesn't get added everytime the contacts window
			 * is opened when the contacts file doesn't exist. */
			m_createdDefault = true;
			CreateDefaultContacts();
		}
		return true;
	}
	
	s_pContacts = new CContactList();

	if ( m_pDocument != NULL )
	{
		if ( m_pDocument->LoadFile( s_szContactsFile ) == true )
		{
			TiXmlHandle		doc_handle( m_pDocument );

			//
			//	Parse the contacts
			//

			// Get the contact element
			TiXmlElement * p_contacts(doc_handle.FirstChild( "contacts" ).FirstChild("contact").Element());
			// If p_contacts isn't null, loop through each contact element adding it to the s_pContacts
			if (p_contacts != NULL) {
				int contactCount = 1;
				for ( ; p_contacts != NULL; p_contacts = p_contacts->NextSiblingElement() )
				{
					// Setup vars
					CString sName("");
					CString sAddress("");
					CContactEmailList * p_ContactEmailList;
					CContactPhoneList * p_ContactPhoneList;

					// Get name info (this is mandatory!)
					TiXmlElement * p_name(p_contacts->FirstChildElement("name"));
					if (p_name != NULL && p_name->GetText() != NULL && strlen(p_name->GetText()) > 0) {
						sName = p_name->GetText();
					} else {
						printf("Error loading contacts: Missing name element for contact %d", contactCount);
						return false;
					}

					//Get address info
					TiXmlElement * p_address(p_contacts->FirstChildElement("address"));
					if (p_address != NULL && p_address->GetText() != NULL && strlen(p_address->GetText()) > 0) {
						sAddress = p_address->GetText();
					}
					else
					{
					 	sAddress = "";
					}

					//Get email info
					int emailCount = 1;
					p_ContactEmailList = new CContactEmailList();
					
					TiXmlElement * p_email(p_contacts->FirstChildElement("email"));
					if (p_email != NULL) {
						for ( ; p_email != NULL; p_email = p_email->NextSiblingElement("email") )
						{
							CString sEmailName("");
							CString sEmailAddress("");

							if (p_email->Attribute("name") != NULL && p_email->GetText() != NULL) {
								sEmailName = p_email->Attribute("name");
								sEmailAddress = p_email->GetText();
								CContactEmailEntry *p_newEmailEntry = new CContactEmailEntry(sEmailName, sEmailAddress);
								p_ContactEmailList->push_back(p_newEmailEntry);
							} else {
								printf("Error loading contacts: Contact %d, email entry %d", contactCount, emailCount);
								return false;
							}
							emailCount++;
						}
					} else {
						//printf("Error loading contacts: Contact %d has no emails", contactCount);
					}

					//Get phone info
                    int phoneCount = 1;
                    p_ContactPhoneList = new CContactPhoneList();

                    TiXmlElement * p_phone(p_contacts->FirstChildElement("phone"));
                    if (p_phone != NULL) {
                        for ( ; p_phone != NULL; p_phone = p_phone->NextSiblingElement("phone") )
                        {
                            CString sPhoneName("");
                            CString sPhoneNumber("");

                            if (p_phone->Attribute("name") != NULL && p_phone->GetText() != NULL) {
                                sPhoneName = p_phone->Attribute("name");
                                sPhoneNumber = p_phone->GetText();
                                CContactPhoneEntry *p_newPhoneEntry = new CContactPhoneEntry(sPhoneName, sPhoneNumber);
                                p_ContactPhoneList->push_back(p_newPhoneEntry);
                            } else {
                                printf("Error loading contacts: Contact %d, phone entry %d", contactCount, phoneCount);
                                return false;
                            }
                            phoneCount++;
                        }
                    } else {
                        //printf("Error loading contacts: Contact %d has no phones", contactCount);
                    }

					// Build the entry and add it to contacts
					CContactEntry *newEntry = new CContactEntry(sName, sAddress, p_ContactEmailList, p_ContactPhoneList);
					CContacts::AddEntry(newEntry);

					contactCount++;
				}
			} else {
				//printf( "Error loading contacts: Could not find contacts element");
			}
		}
		else
		{
			printf("Error loading contacts: Failed to find file or invalid XML!" );
			return false;
		}
	}
	return true;
}

//******************************************************************************
//
//******************************************************************************
void	CContacts::Save()
{
	//
	//	Now save all the changes to the contacts file
	//
	if ( m_pDocument != NULL )
	{
		TiXmlHandle doc_handle(m_pDocument);
		
		TiXmlElement * p_contacts(doc_handle.FirstChild("contacts").Element());
		
		// Check to make sure the document isn't blank
		if ( p_contacts == NULL )
		{
			// It's blank, so the file didn't exist on startup, let's build a
			// blank contacts xml document
			PopulateXmlDocument(m_pDocument);
			p_contacts = doc_handle.FirstChild("contacts").Element();
		}
		
		if ( p_contacts != NULL )
		{
			// Remove all previous children
			p_contacts->Clear();

			// Iterate through all of the contacts, create elements and add to contacts element
			for (CContactList::iterator it = s_pContacts->begin(); it != s_pContacts->end(); ++it)
			{			
				TiXmlElement* p_contact = new TiXmlElement("contact");
				
				TiXmlElement* p_name = new TiXmlElement("name");
				p_name->LinkEndChild(new TiXmlText((const char *)(*it)->GetName()));
				p_contact->LinkEndChild(p_name);
				
				CContacts::CContactEmailList::iterator itEmail = (*it)->GetEmailList()->begin();
				while (itEmail != (*it)->GetEmailList()->end())
				{
					TiXmlElement* p_email = new TiXmlElement("email");
					p_email->SetAttribute("name", (const char *)(*itEmail)->GetName());
					p_email->LinkEndChild(new TiXmlText((const char *) (*itEmail)->GetEmail()));
					p_contact->LinkEndChild(p_email);
					itEmail++;
				}
				
				if ((*it)->GetPhoneList()->size() != 0)
				{
					CContacts::CContactPhoneList::iterator itPhone = (*it)->GetPhoneList()->begin();
					while (itPhone != (*it)->GetPhoneList()->end())
					{
						TiXmlElement* p_phone = new TiXmlElement("phone");
						p_phone->SetAttribute("name", (const char *)(*itPhone)->GetName());
						p_phone->LinkEndChild(new TiXmlText((const char *) (*itPhone)->GetPhone()));
						p_contact->LinkEndChild(p_phone);
						itPhone++;
					}
				}
				
				if ((*it)->GetAddress() != "")
				{
					TiXmlElement* p_address = new TiXmlElement("address");
					p_address->LinkEndChild(new TiXmlText((const char *)(*it)->GetAddress()));
					p_contact->LinkEndChild(p_address);
				}
				
				p_contacts->LinkEndChild(p_contact);
			}
		}

		if ( (CFileSystem::FileExists( s_szContactsFile ) == false) || (CFileSystem::MakeWritable( s_szContactsFile ) == true) )
		{
			if ( m_pDocument->SaveFile( s_szContactsFile ) == false )
			{
				ASSERT( 0, "Failed to save contacts file!" );
			}
		}
	}
}

//******************************************************************************
//	
//******************************************************************************
void	CContacts::Close()
{
	CContacts::Save();

	CContacts::CContactList::iterator it = s_pContacts->begin();
	while (it != s_pContacts->end()) {
		SAFE_DELETE(*it);
		it++;
	}
}

//******************************************************************************
//	
//******************************************************************************
void	CContacts::AddEntry( CContactEntry * const p_var )
{
	if ( s_pContacts == NULL )
	{
		s_pContacts = new CContactList();
	}
	
	// We want to sort the contact list by name, best way to insert them in order
	// To speed things up, lets use a binary insertion!
	CContactList::iterator iBegin  = s_pContacts->begin();
	CContactList::iterator iEnd    = s_pContacts->end();
	CContactList::iterator iSorter = iBegin;
	int                steps   = s_pContacts->size();

	// Find a place to put the contact
	while (iBegin != iEnd)
	{
	  // start with the iterator at the current beginning of the list
	  iSorter = iBegin;
	  // find the middle
	  steps = (int)(steps / 2);
	  // move the iterator to the middle
	  for (int i = 0; i < steps; i++)
	  {
		 ++iSorter;
	  }
	  
	  // use the > of CString to see if the name is greater than the iterator name
	  // if so move the beginning up to the new middle and restart
	  // 
	  if (p_var->GetName() > (*iSorter)->GetName())
	  {
		 // change the beginning of the list to the current  
		 // iterator position
		 iBegin = iSorter;
		 // if we didn't move at all, and if we aren't at the  
		 // end of the list,  move the beginning one more step.
		 if (steps == 0 && iBegin != iEnd)
		 {
			++iBegin;
		 }
		 // we need to do it this way because eventually, you just 
		 // run out of "steps" (it's equal to 0), and the routine 
		 // would just sit there on the same iterator forever.  If 
		 // it gets to this point, it's safe to assume that simply 
		 // moving the iterator one more step in the appropriate 
		 // direction will locate the correct insertion point.
	  }
	  else
	  {
		 iEnd = iSorter;
		 if (steps == 0 && iEnd != iBegin)
		 {
			--iEnd;
		 }
	  }
	}
	iSorter = iEnd;
	s_pContacts->insert(iSorter, p_var);
	
}

//******************************************************************************
//	
//******************************************************************************
void	CContacts::CreateDefaultContacts()
{
	CContactEmailList* emailList = new CContactEmailList();
	emailList->push_back(new CContactEmailEntry("gmail", "psposte@gmail.com"));
	
	AddEntry(new CContactEntry("PSPoste team", "", emailList, NULL));
}

//******************************************************************************
//	
//******************************************************************************
void	CContacts::PopulateXmlDocument(TiXmlDocument* document)
{
	TiXmlElement* p_contactsElement = new TiXmlElement("contacts");
	document->LinkEndChild(p_contactsElement);
}

//*******************************  END OF FILE  ********************************
