/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlAddress.cpp

  Description :	

  Last Modified $Date: 2007/01/01 21:55:47 $

  $Revision: 1.5 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControlAddress.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFrameWork.h"
#include "CControlAddress.h"
#include "CMath.h"
#include "CHUHelp.h"
#include "CContacts.h"
#include "CLog.h"
#include "CAssert.h"
#include "CKeyboardNavWrapper.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const float			WINDOW_BORDER(2.f); // Border around context menu (all sides)
static const V2				WINDOW_SIZE(CGfx::s_ScreenWidth, CGfx::s_ScreenHeight);
static const V2				WINDOW_POSITION(0, 0);
static const float			WINDOW_ITEM_HEIGHT(17.f);
static const float			MID_BAR_WIDTH(2.f);
static const float			WINDOW_ITEM_PADDING(2.f); // Vert padding between menu items
static const unsigned int	MAX_DISPLAYED_ITEMS(13);
static const float			INACTIVE_BORDER(1.f);
static const float			INACTIVE_PADDING(1.f);
static const float			WINDOW_ITEM_WIDTH((WINDOW_SIZE.x - WINDOW_BORDER*2 - WINDOW_ITEM_PADDING * 4 - MID_BAR_WIDTH)/2);
static const float			HEADER_HEIGHT(16.f);

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControlAddress::CControlAddress(V2 const size, V2 const position) :
	m_size(size),
	m_position(position),
	m_pAvailAddressList(NULL),
	m_pAvailDisplayAddressList(NULL),
	m_pAvailSelectedItem(NULL),
	m_pAvailDisplayedItem(NULL),
	m_pAvailScrolling(NULL),
	m_pSelectAddressList(NULL),
	m_pSelectDisplayAddressList(NULL),
	m_pSelectSelectedItem(NULL),
	m_pSelectDisplayedItem(NULL),
	m_pSelectScrolling(NULL),
	m_pDialogBox(NULL),
	m_pContextMenu(NULL),
	m_szAddresses(""),
	m_selectedList(LIST_SELECTED) //set to LIST_AVAILABLE if there is in fact entries in contacts
{
    // Build the available list from CContacts
	m_pAvailAddressList = new AddressEntryList();
	m_pAvailDisplayAddressList = new AddressEntryList();

	CContacts::CContactList::iterator it = CContacts::s_pContacts->begin();
	CString text("");
	while (it != CContacts::s_pContacts->end()) {
        CContacts::CContactEmailList::iterator itEmail = (*it)->GetEmailList()->begin();
        while (itEmail != (*it)->GetEmailList()->end()) {
			text.Printf("%s (%s)", (const char *)(*it)->GetName(), (const char *)(*itEmail)->GetName());
            InputBoxAddressEntry *newEntry = new InputBoxAddressEntry(text, (*itEmail)->GetEmail());
            m_pAvailAddressList->push_back(newEntry);
			itEmail++;
        }
		it++;
    }

	if (m_pAvailAddressList->size() != 0) {
		m_pAvailSelectedItem = *(m_pAvailAddressList->begin());
		m_pAvailDisplayedItem = m_pAvailSelectedItem;
		buildDisplayList(m_pAvailAddressList, m_pAvailDisplayAddressList, &m_pAvailDisplayedItem);
		m_pAvailScrolling = new CScrollingText(m_pAvailSelectedItem->GetName());
		m_selectedList = LIST_AVAILABLE;
	}

	// Build the selected list (empty to start)
	m_pSelectAddressList = new AddressEntryList();
	m_pSelectDisplayAddressList = new AddressEntryList();
	m_pSelectSelectedItem = NULL;
	m_pSelectDisplayedItem = NULL;

	m_pSingleEntryEmailList = new AddressEntryList();

	m_pSelectScrolling = new CScrollingText("");
	m_pAddressScrolling = new CScrollingText("");
}

//******************************************************************************
//	
//******************************************************************************
CControlAddress::~CControlAddress()
{
	AddressEntryList::iterator it;

	// Delete the address items
	it = m_pAvailAddressList->begin();
	while( it !=  m_pAvailAddressList->end() ) {
		SAFE_DELETE(*it);
		it++;
	}
	
	// Delete the single entry items
	it = m_pSingleEntryEmailList->begin();
	while( it !=  m_pSingleEntryEmailList->end() ) {
		SAFE_DELETE(*it);
		it++;
	}

	// Remove the lists
	SAFE_DELETE(m_pAvailAddressList);
	SAFE_DELETE(m_pAvailDisplayAddressList);
	SAFE_DELETE(m_pSelectAddressList);
	SAFE_DELETE(m_pSelectDisplayAddressList);
	SAFE_DELETE(m_pSingleEntryEmailList);

	// Remove the scrolling texts
	SAFE_DELETE(m_pAvailScrolling);
	SAFE_DELETE(m_pSelectScrolling);
	SAFE_DELETE(m_pAddressScrolling);
}	

//******************************************************************************
//	
//******************************************************************************
void	CControlAddress::RenderControl()
{
	if(isHidden()) {
		return;
	}

	AddressEntryList::iterator it;
	
	// Build the address text
	if (m_pSelectDisplayAddressList->size() > 0) {
		it = m_pSelectAddressList->begin();
		m_szAddresses.Printf("%s", (const char *)(*it)->GetName());
		it++;

		while( it !=  m_pSelectAddressList->end() ) {
			m_szAddresses.Printf("%s, %s", (const char *)m_szAddresses, (const char *)(*it)->GetName());
			it++;
		}
	} else {
		m_szAddresses.Printf("");
	}

	// Handle scrolling text fields
	if (m_pAvailSelectedItem != NULL) {
		m_pAvailScrolling->updateText(m_pAvailSelectedItem->GetName());
	}
	if (m_pSelectSelectedItem != NULL) {
		m_pSelectScrolling->updateText(m_pSelectSelectedItem->GetName());
	}
	m_pAddressScrolling->updateText(m_szAddresses);

	//Render Active
	if (isActive()) {

		// Draw the borders
		// Sort of cheating here, couldn't send bgcolor to none opaque, if we want that functionality
		// need to draw 4 seperate quads
		CGfx::DrawQuad( WINDOW_POSITION,
						WINDOW_SIZE,
						CSkinManager::GetColor("address_box", "border_color", 0xFFFFFFFF));

		// Draw menu background (overlay over the menu border quad)
		CGfx::DrawQuad( V2( WINDOW_POSITION.x + WINDOW_BORDER, WINDOW_POSITION.x + WINDOW_BORDER),
						V2( WINDOW_SIZE.x - WINDOW_BORDER * 2, WINDOW_SIZE.y - WINDOW_BORDER * 2),
						CSkinManager::GetColor( "address_box", "background_color", 0xFF000000));

		// Draw headers
		CGfx::DrawQuad( V2(WINDOW_POSITION.x + WINDOW_BORDER, WINDOW_POSITION.y + WINDOW_BORDER),
						V2(WINDOW_SIZE.x - WINDOW_BORDER*2, HEADER_HEIGHT),
						CSkinManager::GetColor( "address_box", "header_color", 0xFF000000));

		CString header1Text("Available Addresses");
		V2 head1Size(CFont::GetActiveUiFont()->GetStringSize(header1Text));

		CFont::GetActiveUiFont()->Print(header1Text, 
									   V2(WINDOW_POSITION.x + WINDOW_BORDER + ((WINDOW_SIZE.x - WINDOW_BORDER * 3)/2)/2 - head1Size.x/2,
									      WINDOW_POSITION.y + WINDOW_BORDER + HEADER_HEIGHT/2 - head1Size.y/2),
									   CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));

		CString header2Text("Selected Addresses");
		V2 head2Size(CFont::GetActiveUiFont()->GetStringSize(header2Text));

		CFont::GetActiveUiFont()->Print(header2Text, 
									   V2(WINDOW_POSITION.x + WINDOW_SIZE.x - WINDOW_BORDER - ((WINDOW_SIZE.x - WINDOW_BORDER * 3)/2)/2 - head2Size.x/2,
									      WINDOW_POSITION.y + WINDOW_BORDER + HEADER_HEIGHT/2 - head2Size.y/2),
									   CSkinManager::GetColor( "address_box", "header_text_color", 0xFFFFFFFF));

		// Draw middle seperating bar
		CGfx::DrawQuad( V2( WINDOW_POSITION.x + WINDOW_SIZE.x/2 - MID_BAR_WIDTH/2, WINDOW_POSITION.y),
						V2( MID_BAR_WIDTH, WINDOW_SIZE.y),
						CSkinManager::GetColor( "address_box", "border_color", 0xFFFFFFFF));

		// Draw line under header
		CGfx::DrawQuad( V2(WINDOW_POSITION.x, WINDOW_POSITION.y + WINDOW_BORDER + HEADER_HEIGHT),
						V2(WINDOW_SIZE.x, WINDOW_BORDER),
						CSkinManager::GetColor( "address_box", "border_color", 0xFFFFFFFF));
		


		// Initialize the text position coordinates
		float textPositionX = WINDOW_POSITION.x + WINDOW_BORDER + WINDOW_ITEM_PADDING;
		float textPositionY = WINDOW_POSITION.y + WINDOW_BORDER * 2 + HEADER_HEIGHT + WINDOW_ITEM_PADDING;
		

		// Print the text for each available address
		it = m_pAvailDisplayAddressList->begin();
		while( it !=  m_pAvailDisplayAddressList->end() ) {
			textPositionY += WINDOW_ITEM_PADDING / 2;
			V2 text_size( CFont::GetActiveUiFont()->GetStringSize( (*it)->GetName() ));
			V2 text_position( textPositionX, textPositionY + WINDOW_ITEM_HEIGHT - text_size.y );
			
			// Print regular text for unselected, print selected text and box for selected item
			if ((*it) != m_pAvailSelectedItem || m_selectedList == LIST_SELECTED) {
				CGfx::ClearClipRegions();
				CGfx::SetClipRegion( text_position, V2(WINDOW_ITEM_WIDTH, text_size.y));
				CFont::GetActiveUiFont()->Print( (*it)->GetName(), text_position, CSkinManager::GetColor( "address_box", "text_color", 0xFFFFFFFF));
				CGfx::DisableClipRegions();
			} else {
				CGfx::DrawQuad( V2( textPositionX - WINDOW_ITEM_PADDING, textPositionY - WINDOW_ITEM_PADDING / 2 ),
								V2( (WINDOW_SIZE.x - WINDOW_BORDER * 2 - MID_BAR_WIDTH)/2, WINDOW_ITEM_HEIGHT + WINDOW_ITEM_PADDING ),
								CSkinManager::GetColor( "address_box", "selected_item_hilight_color", 0xFFFFFFFF));
				m_pAvailScrolling->Render(WINDOW_ITEM_WIDTH, text_position, CSkinManager::GetColor( "address_box", "selected_text_color", 0xFF000000));
			}	

			// Move text position to next item
			textPositionY += WINDOW_ITEM_HEIGHT + WINDOW_ITEM_PADDING / 2;	
			it++;
		}

		// Print the text for each selected address
		it = m_pSelectDisplayAddressList->begin();
		textPositionX = WINDOW_SIZE.x / 2 + MID_BAR_WIDTH/2 + WINDOW_ITEM_PADDING;
		textPositionY = WINDOW_POSITION.y + WINDOW_BORDER * 2 + HEADER_HEIGHT + WINDOW_ITEM_PADDING;
		while( it !=  m_pSelectDisplayAddressList->end() ) {
			textPositionY += WINDOW_ITEM_PADDING / 2;
			V2 text_size( CFont::GetActiveUiFont()->GetStringSize( (*it)->GetName() ));
			V2 text_position(textPositionX, textPositionY + WINDOW_ITEM_HEIGHT - text_size.y );
			
			// Print regular text for unselected, print selected text and box for selected item
			if ((*it) != m_pSelectSelectedItem || m_selectedList == LIST_AVAILABLE) {
				CGfx::ClearClipRegions();
				CGfx::SetClipRegion( text_position, V2(WINDOW_ITEM_WIDTH, text_size.y));
				CFont::GetActiveUiFont()->Print( (*it)->GetName(), text_position, CSkinManager::GetColor( "address_box", "text_color", 0xFFFFFFFF));
				CGfx::DisableClipRegions();
			} else {
				CGfx::DrawQuad( V2( textPositionX - WINDOW_ITEM_PADDING, textPositionY - WINDOW_ITEM_PADDING / 2 ),
								V2( (WINDOW_SIZE.x - WINDOW_BORDER * 2 - MID_BAR_WIDTH)/2, WINDOW_ITEM_HEIGHT + WINDOW_ITEM_PADDING ),
								CSkinManager::GetColor( "address_box", "selected_item_hilight_color", 0xFFFFFFFF));
				m_pSelectScrolling->Render(WINDOW_ITEM_WIDTH, text_position, CSkinManager::GetColor( "address_box", "selected_text_color", 0xFF000000));
			}	

			// Move text position to next item
			textPositionY += WINDOW_ITEM_HEIGHT + WINDOW_ITEM_PADDING / 2;	
			it++;
		}
	} else { // Render inactive
		// Draw the borders
		// Sort of cheating here, couldn't send bgcolor to none opaque, if we want that functionality
		// need to draw 4 seperate quads
		if (isSelected()) {
			CGfx::DrawQuad( V2( m_position.x, m_position.y ),
							V2( m_size.x, m_size.y),
							CSkinManager::GetColor( "address_box", "inactive_selected_border_color", 0xFFAAFFAA));
		} else {
			CGfx::DrawQuad( V2( m_position.x, m_position.y ),
							V2( m_size.x, m_size.y),
							CSkinManager::GetColor( "address_box", "inactive_border_color", 0xFFAAFFAA));
		}

		// Draw menu background (overlay over the menu border quad)
		CGfx::DrawQuad( V2( m_position.x + INACTIVE_BORDER, m_position.y + INACTIVE_BORDER),
						V2( m_size.x - INACTIVE_BORDER * 2, m_size.y - INACTIVE_BORDER * 2),
						CSkinManager::GetColor( "address_box", "inactive_background_color", 0xFF00FF00));

		// Draw text
		V2 text_size( CFont::GetActiveUiFont()->GetStringSize( m_szAddresses ));
		V2 text_position( m_position.x + INACTIVE_BORDER + INACTIVE_PADDING, m_position.y + m_size.y - INACTIVE_BORDER - INACTIVE_PADDING - text_size.y );
		
		if (isSelected()) {
			m_pAddressScrolling->Render(m_size.x - INACTIVE_BORDER * 2 - INACTIVE_PADDING * 2, text_position, CSkinManager::GetColor( "address_box", "inactive_text_color", 0xFFFFFFFF));
		} else {
			m_pAddressScrolling->Render(m_size.x - INACTIVE_BORDER * 2 - INACTIVE_PADDING * 2, text_position, CSkinManager::GetColor( "address_box", "inactive_text_color", 0xFFFFFFFF), false);
		}
	}
}

//******************************************************************************
//
//******************************************************************************
void	CControlAddress::ProcessControl() {

	AddressEntryList::iterator it;

	if (isActive()) {
		switch (m_selectedList) {
			case LIST_AVAILABLE:
				CHUHelp::Clear();
				CHUHelp::SetButton(CHUHelp::BUTTON_UP, "Next Item");
				CHUHelp::SetButton(CHUHelp::BUTTON_DOWN, "Previous Item");
				CHUHelp::SetButton(CHUHelp::BUTTON_RIGHT, "Switch to selected address list");
				CHUHelp::SetButton(CHUHelp::BUTTON_CROSS, "Add address to selected address list");
				CHUHelp::SetButton(CHUHelp::BUTTON_TRIANGLE, "Open context menu (including add single entry email)");
				break;
			case LIST_SELECTED:
				CHUHelp::Clear();
				CHUHelp::SetButton(CHUHelp::BUTTON_UP, "Next Item");
				CHUHelp::SetButton(CHUHelp::BUTTON_DOWN, "Previous Item");
				CHUHelp::SetButton(CHUHelp::BUTTON_LEFT, "Switch to available address list");
				CHUHelp::SetButton(CHUHelp::BUTTON_CROSS, "Remove address from selected address list");
				CHUHelp::SetButton(CHUHelp::BUTTON_TRIANGLE, "Open context menu (including add single entry email)");
				break;
		}
	}
	// Handle context menu returning
	if (m_pContextMenu != NULL) {
        if ( m_pContextMenu->getExitValue() != -1 ) {
            //Menu has return a value, process it see eMenuItems enum
            switch(m_pContextMenu->getExitValue()) {
                case CONTEXT_HIDE:
                    break;
                case CONTEXT_CLOSE:
					deActivate();
                    break;
                case CONTEXT_SINGLE_USE_EMAIL:
					m_pDialogBox = new CDialogBoxInput(250, V2(-1, -1), "Please enter an email address", true, true);
                    break;
                default:
                    break;
            }
            //Remove the context menu
            SAFE_DELETE(m_pContextMenu); //sets to NULL so input processing appears again
        }
    }

	// Handle dialog box returning
	if (m_pDialogBox != NULL) {
		if (m_pDialogBox->getExitValue() != -1) {
			switch(m_pDialogBox->getExitValue()) {
				case CDialogBoxInput::OK:
					// handle addition of single use email
            		InputBoxAddressEntry *newEntry = new InputBoxAddressEntry(m_pDialogBox->getReturnText(), m_pDialogBox->getReturnText());
					m_pSelectSelectedItem = newEntry;
					AddToSelectedList();
					m_pSingleEntryEmailList->push_back(newEntry); // keep track of single entries for deletion purposes
					break;
				case CDialogBoxInput::CANCEL:
					break;
			}
			//Remove the dialog box
			SAFE_DELETE(m_pDialogBox);
		}
	}
}

//******************************************************************************
//
//******************************************************************************
void	CControlAddress::ProcessControlInput()
{
	AddressEntryList::iterator it;

	if (isActive() && m_pContextMenu == NULL && m_pDialogBox == NULL ) {
		// Handle available addresses navigation
		if (m_selectedList == LIST_AVAILABLE) {
			if (m_pAvailAddressList->size() != 0) {
				// Shift selected item up one unless already first in list
				if (CKeyboardNavWrapper::Up()) {
					it = m_pAvailAddressList->begin();
					if (m_pAvailSelectedItem != (*it)) {
						while( it !=  m_pAvailAddressList->end() ) {
							if (m_pAvailSelectedItem == (*it)) {
								it--;
								m_pAvailSelectedItem = (*it);
								updateFirstDisplayed(m_pAvailAddressList, m_pAvailDisplayAddressList, &m_pAvailSelectedItem, &m_pAvailDisplayedItem);
								return;
							}
							it++;
						}
					}
				}

				// Shift selected item down one unless already last in list
				if (CKeyboardNavWrapper::Down()) {
					it = m_pAvailAddressList->begin();
					while( it !=  m_pAvailAddressList->end() ) {
						if (m_pAvailSelectedItem == (*it)) {
							if (++it != m_pAvailAddressList->end()) {
								m_pAvailSelectedItem = (*it);
								updateFirstDisplayed(m_pAvailAddressList, m_pAvailDisplayAddressList, &m_pAvailSelectedItem, &m_pAvailDisplayedItem);
								return;
							}
							continue; //already incremented
						}
						it++;
					}
				}
				if (CKeyboardNavWrapper::Right()) {
					if(m_pSelectAddressList->size() != 0) {
						m_selectedList = LIST_SELECTED;
					}
				}
				if (CKeyboardNavWrapper::ActivateControl()) {
					m_pSelectSelectedItem = m_pAvailSelectedItem;
					AddToSelectedList();					
				}
			}
		}

		// Handle selected addresses navigation
		if (m_selectedList == LIST_SELECTED) {
			if (m_pSelectAddressList->size() != 0) {
				// Shift selected item up one unless already first in list
				if (CKeyboardNavWrapper::Up()) {
					it = m_pSelectAddressList->begin();
					if (m_pSelectSelectedItem != (*it)) {
						while( it !=  m_pSelectAddressList->end() ) {
							if (m_pSelectSelectedItem == (*it)) {
								it--;
								m_pSelectSelectedItem = (*it);
								updateFirstDisplayed(m_pSelectAddressList, m_pSelectDisplayAddressList, &m_pSelectSelectedItem, &m_pSelectDisplayedItem);
								return;
							}
							it++;
						}
					}
				}
	
				// Shift selected item down one unless already last in list
				if (CKeyboardNavWrapper::Down()) {
					it = m_pSelectAddressList->begin();
					while( it !=  m_pSelectAddressList->end() ) {
						if (m_pSelectSelectedItem == (*it)) {
							if (++it != m_pSelectAddressList->end()) {
								m_pSelectSelectedItem = (*it);
								updateFirstDisplayed(m_pSelectAddressList, m_pSelectDisplayAddressList, &m_pSelectSelectedItem, &m_pSelectDisplayedItem);
								return;
							}
							continue; //already incremented
						}
						it++;
					}
				}
				if (CKeyboardNavWrapper::Left()) {
					if(m_pAvailAddressList->size() != 0) {
						m_selectedList = LIST_AVAILABLE;
					}
				}	
				if (CKeyboardNavWrapper::ActivateControl()) {
					DeleteFromSelectedList();
				}
			}
		}
		
		// Handle context menu creation
		if (CKeyboardNavWrapper::OpenContextMenu()) {
			// Buld the context menu
            CContextMenu::CContextMenuItemList * itemList = new CContextMenu::CContextMenuItemList();

			itemList->push_back(new CContextMenuItem("Hide Menu", CONTEXT_HIDE, true));
			itemList->push_back(new CContextMenuItem("Add Single Use Email", CONTEXT_SINGLE_USE_EMAIL, true)); 
            itemList->push_back(new CContextMenuItem("Close", CONTEXT_CLOSE, false));

            // Create the context menu
            m_pContextMenu = new CContextMenu(itemList, new V2(10, 10));
		}
	}
}

//******************************************************************************
// This method assumes that the m_pDisplayedList is properly set, and has 
// enough items underneath to properly display a full select menu
//******************************************************************************
void	CControlAddress::buildDisplayList(AddressEntryList *list, AddressEntryList *displayList, InputBoxAddressEntry **firstDisplayed)
{
	// Remove existing items from the list
	displayList->clear();

	if (list->size() == 0) {
		*firstDisplayed = NULL;
	}

	// Build the list of items to be displayed, starting with the item in m_pDisplayedItem
	AddressEntryList::iterator it = list->begin();
	while( it != list->end() ) {
		// Found the displayed item, display the next items
		if (*firstDisplayed == (*it)) {
			//Ensure doesn't reach end (this will happen if list.size() < max items
			for(unsigned int x=0; (x<MAX_DISPLAYED_ITEMS && it != list->end()); x++) {
				displayList->push_back(*it);
				it++;
			}
			return;
		}
		it++;
	}
}

//******************************************************************************
// 
//******************************************************************************
void	CControlAddress::updateFirstDisplayed(AddressEntryList *list, AddressEntryList *displayList, InputBoxAddressEntry **selected, InputBoxAddressEntry **firstDisplayed) {

	AddressEntryList::iterator it;

	if (selected == NULL || firstDisplayed == NULL) {
		return;
	}

	if (list->size() == 0) {
		*firstDisplayed = NULL;
		return;
	}

	// Don't change if selected item is already in the list
	it = displayList->begin();
	while( it !=  displayList->end() ) {
		if (*selected == (*it)) {
			return;
		}
		it++;
	}

	// If selected item found before first display, first display set to selected item and rebuild list
	it = list->begin();
	while( it !=  list->end() ) {
		if ((*it) == *firstDisplayed) break; // selected item must be after list
		if ((*it) == *selected) {
			*firstDisplayed = *selected;
			buildDisplayList(list, displayList, firstDisplayed);
			return;	
		}
		it++;
	}
                            
	// selected must be after
	it = list->begin();
	while( it !=  list->end() ) {
		if ((*it) == *firstDisplayed) {
			*firstDisplayed = *(++it); // Move displayed up by one
			buildDisplayList(list, displayList, firstDisplayed);
			updateFirstDisplayed(list, displayList, selected, firstDisplayed); // re run to make sure actually in list!
			return;
		}
		it++;
	}
}


//******************************************************************************
// Takes the value in m_pSelectSelectedAddress and adds it to the selected list
// and rebuilds the selected display list accordingly
//******************************************************************************
void	CControlAddress::AddToSelectedList() {
	// Verify not already in list
	AddressEntryList::iterator it = m_pSelectAddressList->begin();
	while( it !=  m_pSelectAddressList->end() ) {
		if ((*it) == m_pSelectSelectedItem) {
			return;
		}
		it++;
	}

	// No other items in the list, need to initialize display list as well
	if (m_pSelectAddressList->size() == 0) {
		m_pSelectAddressList->push_back(m_pSelectSelectedItem);
		m_pSelectDisplayAddressList->push_back(m_pSelectSelectedItem);
		m_pSelectDisplayedItem = m_pSelectSelectedItem; 
		buildDisplayList(m_pSelectAddressList, m_pSelectDisplayAddressList, &m_pSelectDisplayedItem);
	} else {
		m_pSelectAddressList->push_back(m_pSelectSelectedItem);
		if (m_pSelectDisplayAddressList->size() < MAX_DISPLAYED_ITEMS ) {
			m_pSelectDisplayAddressList->push_back(m_pSelectSelectedItem);
		}
		updateFirstDisplayed(m_pSelectAddressList, m_pSelectDisplayAddressList, &m_pSelectSelectedItem, &m_pSelectDisplayedItem);
	}
}

//******************************************************************************
// Remove the selected item from the selected display list and clean up the lists
//******************************************************************************
void	CControlAddress::DeleteFromSelectedList() {
	InputBoxAddressEntry *p_addressToDelete = m_pSelectSelectedItem;

	// Only a single item in the list
	if (m_pSelectAddressList->size() == 1) {
		m_pSelectAddressList->erase(m_pSelectAddressList->begin());
		m_pSelectDisplayAddressList->erase(m_pSelectDisplayAddressList->begin());
		m_pSelectSelectedItem = NULL;
		m_pSelectDisplayedItem = NULL;
		m_selectedList = LIST_AVAILABLE;
		return;
	}

	// Multiple items in the list, need to remove from select and display
	AddressEntryList::iterator it = m_pSelectAddressList->begin();
	while(it !=  m_pSelectAddressList->end())
	{
		if ((*it) == p_addressToDelete)
		{
			// Move the selected to the next item
			it++;
			if (it == m_pSelectAddressList->end()) {
				// No items past this one, move to the previous item instead
				it--;
				it--;
				m_pSelectSelectedItem = (*it);
				it++;
				m_pSelectAddressList->erase(it);
			} else {
				m_pSelectSelectedItem = (*it);
				it--;
				m_pSelectAddressList->erase(it);
			}

			//check if the removed item was the first displayed, if so set the first displayed to the newly selected item
			if (m_pSelectDisplayedItem == p_addressToDelete) {
				m_pSelectDisplayedItem = m_pSelectSelectedItem;
			}
			break;
		}				   	
		it++;
	}		

	//Rebuild the display list
	buildDisplayList(m_pSelectAddressList, m_pSelectDisplayAddressList, &m_pSelectDisplayedItem);
}

//******************************************************************************
//
//******************************************************************************
void CControlAddress::AddAddress(CContactEntry* contact)
{
	CString text("");
	CContacts::CContactEmailList::iterator itEmail = contact->GetEmailList()->begin();
	while (itEmail != contact->GetEmailList()->end()) {
		text.Printf("%s (%s)", (const char *)contact->GetName(), (const char *) (*itEmail)->GetName());
		break;
	}
	InputBoxAddressEntry* newEntry = new InputBoxAddressEntry(text, (*itEmail)->GetEmail());
	AddAddress(newEntry);
}

void CControlAddress::AddAddress(InputBoxAddressEntry* entry) {
	m_pSelectSelectedItem = entry;
	AddToSelectedList();
}

//******************************************************************************
//
//******************************************************************************
CControlAddress::AddressEntryList* const CControlAddress::getAddressList()
{
	return m_pSelectAddressList;
}

//*******************************  END OF FILE  ********************************
