/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CControlTextBox.cpp

  Description :	

  Last Modified $Date: 2007/01/27 18:05:50 $

  $Revision: 1.9 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CControlTextBox.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CInput.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CFrameWork.h"
#include "CControl.h"
#include "CHUHelp.h"
#include "CConfigFile.h"
#include "CKeyboardNavWrapper.h"
#include <psprtc.h>

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const float	WINDOW_BORDER( 1.f ); // Border around context menu (all sides)
static const float  WINDOW_PADDING( 1.f ); // Padding between border and menu item (all sides)
static const char DEFAULT_PASSWORD_CHAR = '*'; // Character to use for password character hidding

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CControlTextBox::CControlTextBox(V2 const size, V2 const position, CText* text) :
	m_size(size),
	m_position(position),
	m_text(text),
	m_passwordText(NULL),
	m_show_cursor(false),
	m_password(false),
	m_passwordChar(DEFAULT_PASSWORD_CHAR)
{
	u64 curTick;
	sceRtcGetCurrentTick(&curTick);
	sceRtcTickAddTicks(&m_next_toggle, &curTick, CURSOR_FLASH_TICK);
}

//******************************************************************************
//	
//******************************************************************************
CControlTextBox::~CControlTextBox()
{
	SAFE_DELETE(m_text);
}	

//******************************************************************************
//	
//******************************************************************************
void	CControlTextBox::RenderControl()
{	
	if(isHidden()) {
		return;
	}

	// Draw the borders
    // Sort of cheating here, couldn't send bgcolor to none opaque, if we want that functionality
	// need to draw 4 seperate quads
	ARGB borderColor;
	if (isSelected())
		borderColor = CSkinManager::GetColor( "text_box", "selected_border_color", 0xFFAAFFAA);
	else
		borderColor = CSkinManager::GetColor( "text_box", "border_color", 0xFFAAFFAA);
	
	CGfx::DrawQuad( V2( m_position.x, m_position.y ),
					V2( m_size.x, m_size.y),
					borderColor);

	// Draw menu background (overlay over the menu border quad)
	ARGB backgroundColor;
	if (isEnabled())
		backgroundColor = CSkinManager::GetColor( "text_box", "background_color", 0xFF00FF00);
	else
		backgroundColor = CSkinManager::GetColor( "text_box", "disabled_background_color", 0xFF00FF00);
		
	CGfx::DrawQuad( V2( m_position.x + WINDOW_BORDER, m_position.y + WINDOW_BORDER),
					V2( m_size.x - WINDOW_BORDER * 2, m_size.y - WINDOW_BORDER * 2),
					backgroundColor);

	// Draw text
	//V2 text_size( CFont::GetDefaultFont()->GetStringSize( m_text ));
	float text_height = CFont::GetActiveTextFont()->GetHeight();
    //V2 text_position( m_position.x + WINDOW_BORDER + WINDOW_PADDING, m_position.y + m_size.y - WINDOW_BORDER - WINDOW_PADDING - text_size.y );
    V2 text_position( m_position.x + WINDOW_BORDER + WINDOW_PADDING, m_position.y + m_size.y - WINDOW_BORDER - WINDOW_PADDING - text_height );
    V2 text_area( m_size.x - WINDOW_BORDER * 2 - WINDOW_PADDING * 2, m_size.y - WINDOW_BORDER * 2 - WINDOW_PADDING * 2 );
	
	u64 curTick;
	sceRtcGetCurrentTick(&curTick);
	if (isActive())
	{
		if (sceRtcCompareTick(&curTick, &m_next_toggle) >= 0)
		{
			m_show_cursor ^= 1;
			sceRtcTickAddTicks(&m_next_toggle, &curTick, CURSOR_FLASH_TICK);
		}
	}
	else
		m_show_cursor = false;
	
	CGfx::SetClipRegion( text_position, text_area );
	//CFont::GetDefaultFont()->Print( m_text, text_position, CSkinManager::GetColor( "text_box", "text_color", 0xFFFFFFFF) );
	
	if (m_password)
		CFont::GetActiveTextFont()->Print(m_passwordText, 0, text_area, text_position, CSkinManager::GetColor( "text_box", "text_color", 0xFFFFFFFF), m_show_cursor);
	else 
		CFont::GetActiveTextFont()->Print(m_text, 0, text_area, text_position, CSkinManager::GetColor( "text_box", "text_color", 0xFFFFFFFF), m_show_cursor);
	
	
	CGfx::DisableClipRegions();

	// Draw cursor
	if (isActive()) {
		// Set up HUD
		CHUHelp::Clear();
		CHUHelp::SetButton(CHUHelp::BUTTON_RTRIGGER, "Toggle Cursor/Entry Mode");
		CHUHelp::SetButton(CHUHelp::BUTTON_LTRIGGER, "Exit input");
	}
}

//******************************************************************************
//
//******************************************************************************
void	CControlTextBox::ProcessControlInput()
{
	if (isActive()) {
		CInput::EnableKeyboard();
		if (CInput::IsKeyboardPressed()) {
			char chr = CInput::GetKeyboardCharKey();
			u16 ctrl = CInput::GetKeyboardControlKey();
			
			if (ctrl == CKeyboard::BACKSPACE)
			{
	   	  		m_text->backspace();
	   	  		if (m_password)
	   	  			m_passwordText->backspace();
			}
			else if (ctrl == CKeyboard::LEFT)
			{
				m_text->decCursor();
	   	  		if (m_password)
	   	  			m_passwordText->decCursor();
			}
			else if (ctrl == CKeyboard::RIGHT)
			{
				m_text->incCursor();
	   	  		if (m_password)
	   	  			m_passwordText->incCursor();
			}
			else if (ctrl == CKeyboard::ESC)
			{
				deActivate();
			}
			else if (ctrl == CKeyboard::HOME)
			{
				// BEN TODO: Home/end support of non-line buffered CText
				home();
			}
			else if (ctrl == CKeyboard::END)
			{
				// BEN TODO: Home/end support of non-line buffered CText
				end();
			}
			else if (chr >= ' ' && chr <= '~')
			{
				//m_text += letter;
				m_text->insert(chr);
				if (m_password)
					m_passwordText->insert(m_passwordChar);
			}
		}

		if (CInput::ShouldDisableKeyboard()) {
			// Means the last keyboard process detected that the keyboard should be disabled, so let's deactivate the control
			CInput::DisableKeyboard();
			deActivate();
		}
	}
}

//******************************************************************************
//
//******************************************************************************
CText* CControlTextBox::getText() {
	return m_text;
}

//******************************************************************************
// 
//******************************************************************************
void	CControlTextBox::ProcessControl() {

}

//******************************************************************************
//	
//******************************************************************************
void CControlTextBox::activate()
{
	CInput::ResetKeyboard();
	m_active = true;
}

//******************************************************************************
//	
//******************************************************************************
void CControlTextBox::deActivate()
{
	m_active = false;
}

//******************************************************************************
//	
//******************************************************************************
void CControlTextBox::setPassword(bool password)
{
	m_password = password;
	resetPasswordText();
}

//******************************************************************************
//	
//******************************************************************************
void CControlTextBox::setText(CText* text)
{
	SAFE_DELETE(m_text);
	m_text = text;
	
	// Handle password text so that we don't convert every render
	resetPasswordText();	
}

//******************************************************************************
//	
//******************************************************************************
void CControlTextBox::resetPasswordText()
{
	// Convert m_text to m_passwordText using the char
	if (m_password)
	{
		SAFE_DELETE(m_passwordText);
		m_passwordText = new CText("");
		for (u32 i=0; i<m_text->size(); i++)
			m_passwordText->append(m_passwordChar);
			
		end(); // Ensure cursor is synced between text and password
	}
}

//******************************************************************************
//	
//******************************************************************************
void CControlTextBox::home()
{
	// A rather dumb solution since the CText shouldn't be used directly in the
	// control, but a CTextLineBuf is overkill... should have another CText
	// in between type... or maybe a CTextLineBuf with 1 line?
	for (size_t i = 0; i < m_text->size(); i++)
	{
		m_text->decCursor();
		if (m_password)
			m_passwordText->decCursor();
	}
}

//******************************************************************************
//	
//******************************************************************************
void CControlTextBox::end()
{
	// A rather dumb solution since the CText shouldn't be used directly in the
	// control, but a CTextLineBuf is overkill... should have another CText
	// in between type... or maybe a CTextLineBuf with 1 line?
	for (size_t i = 0; i < m_text->size(); i++)
	{
		m_text->incCursor();
		if (m_password)
			m_passwordText->incCursor();
	}
}

//*******************************  END OF FILE  ********************************
