/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CDirectoryList.cpp

  Description :	

  Last Modified $Date: 2007/01/07 17:08:28 $

  $Revision: 1.8 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CDialogBoxInput.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CGfx.h"
#include "CSkinManager.h"
#include "CProcess.h"
#include "CFrameWork.h"
#include "CControlTextBox.h"
#include "CHUHelp.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************
static const float	WINDOW_BORDER(2.f); // Border around context menu (all sides)
static const float  WINDOW_PADDING(8.f); // Padding between border and menu item (all sides)
static const float	ITEM_PADDING(4.f);
static const float	BUTTON_HEIGHT(20.f);
static const float	BUTTON_WIDTH(60.f);
static const float	MIN_WIDTH_ONE_BUTTON(WINDOW_BORDER*2 + WINDOW_PADDING*2 + BUTTON_WIDTH);
static const float	MIN_WIDTH_TWO_BUTTON(WINDOW_BORDER*2 + WINDOW_PADDING*2 + BUTTON_WIDTH*2 + ITEM_PADDING);

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CDialogBoxInput::CDialogBoxInput(int const width, V2 const menuPosition, CString const text, bool includeTextInput, bool includeCancelButton, CString defaultText, bool cursorAtBeginning):
	m_size(0,0),
	m_position(menuPosition),
	m_text(text),
	m_includeTextInput(includeTextInput),
	m_includeCancelButton(includeCancelButton),
	m_exitValue(-1),
	m_defaultText(defaultText),
	m_pTextField(NULL),
	m_pOKButton(NULL),
	m_pCancelButton(NULL)
{
	CHUHelp::Clear();
	CHUHelp::SetButton(CHUHelp::BUTTON_UP, "Navigate");
	CHUHelp::SetButton(CHUHelp::BUTTON_DOWN, "Navigate");
	CHUHelp::SetButton(CHUHelp::BUTTON_RIGHT, "Navigate");
	CHUHelp::SetButton(CHUHelp::BUTTON_LEFT, "Navigate");
	CHUHelp::SetButton(CHUHelp::BUTTON_CROSS, "Activate Control");

	// Determine horizontal size
	if (m_includeCancelButton) {
		m_size.x = (width<MIN_WIDTH_TWO_BUTTON) ? MIN_WIDTH_TWO_BUTTON : width;
	} else {
		m_size.x = (width<MIN_WIDTH_ONE_BUTTON) ? MIN_WIDTH_ONE_BUTTON : width;
	}

	V2 text_size(0, 0);
	V2 textbox_size(0, 0);
	V2 button_size(BUTTON_WIDTH, BUTTON_HEIGHT);

	if (m_includeTextInput) {
		textbox_size.x = m_size.x - WINDOW_BORDER*2 - WINDOW_PADDING*2;
		textbox_size.y = 20;
	}

	// If text is provided, determine text_size
	if (!m_text.IsEmpty()) {
		m_text = CFont::GetActiveUiFont()->GetWrappedString(m_text, m_size.x - WINDOW_BORDER*2 - WINDOW_PADDING*2);
		text_size = CFont::GetActiveUiFont()->GetStringSize(m_text);
	}

	// Build vertical total size
	m_size.y = WINDOW_BORDER * 2 + WINDOW_PADDING * 2 + button_size.y;
	if (!m_text.IsEmpty()) {
		m_size.y += text_size.y + ITEM_PADDING;
	}	
	if (m_includeTextInput) {
		m_size.y += textbox_size.y + ITEM_PADDING;
	}

	// Center the dialog box if position coord comes as negative
	if (m_position.x == -1) {
		m_position.x = CGfx::s_ScreenWidth / 2 - m_size.x / 2;
	}
	if (m_position.y == -1) {
		m_position.y = CGfx::s_ScreenHeight /2 - m_size.y / 2;
	}

	// Setup item positioning
	V2 item_position(m_position.x + WINDOW_BORDER+WINDOW_PADDING, m_position.y + WINDOW_BORDER+WINDOW_PADDING);
	if (!m_text.IsEmpty()) {
		item_position.y += text_size.y + ITEM_PADDING;
	}

	// Create the text box
	if (m_includeTextInput) {
		V2 textbox_position(item_position.x, item_position.y);
		m_pTextField = new CControlTextBox(textbox_size, textbox_position);
		m_pTextField->setText(new CText(m_defaultText.GetPtr()));
		if (cursorAtBeginning)
			m_pTextField->home();
		item_position.y += textbox_size.y + ITEM_PADDING;
	}

	// Create the buttons
	if (m_includeCancelButton) {
		V2 ok_button_position(m_position.x + WINDOW_BORDER + WINDOW_PADDING + (m_size.x-WINDOW_BORDER*2 - WINDOW_PADDING*2)/4 - BUTTON_WIDTH/2, item_position.y);
		V2 cancel_button_position(m_position.x + WINDOW_BORDER + WINDOW_PADDING  + ((m_size.x-WINDOW_BORDER*2 - WINDOW_PADDING*2)/4)*3 - BUTTON_WIDTH/2, item_position.y);
		m_pOKButton = new CControlButton (button_size, ok_button_position, "OK");
		m_pCancelButton = new CControlButton(button_size, cancel_button_position, "CANCEL");
	} else {
		V2 ok_button_position(m_position.x + m_size.x/2 - button_size.x/2, item_position.y);
		m_pOKButton = new CControlButton (button_size, ok_button_position, "OK");
	}

	// Create navigation between fields
	if (m_includeTextInput) {
		m_pTextField->m_pDown = m_pOKButton;
		m_pOKButton->m_pUp = m_pTextField;
	}
	if (m_includeCancelButton) {
		m_pCancelButton->m_pLeft = m_pOKButton;
		m_pOKButton->m_pRight = m_pCancelButton;
	}
	if (m_includeCancelButton && m_includeTextInput) {
		m_pCancelButton->m_pUp = m_pTextField;
	}

	m_pControlManager = new CControlManager();

	m_pControlManager->addControl(m_pOKButton);

	if (m_includeTextInput) {
		m_pControlManager->addControl(m_pTextField);
		m_pControlManager->selectControl(m_pTextField);
	} else {
		m_pControlManager->selectControl(m_pOKButton);
	}
	if (m_includeCancelButton) {
		m_pControlManager->addControl(m_pCancelButton);
	}

    SetFocus(true);
}

//******************************************************************************
//	
//******************************************************************************
CDialogBoxInput::~CDialogBoxInput()
{
	SAFE_DELETE(m_pControlManager);
}	

//******************************************************************************
//	
//******************************************************************************
void	CDialogBoxInput::Render()
{
	// Draw background
	CGfx::DrawQuad( V2( 0, 0 ), V2( CGfx::s_ScreenWidth, CGfx::s_ScreenHeight), CSkinManager::GetColor( "dialog_box", "overlay_color", 0x8F000000));

	// Draw the borders
    // Sort of cheating here, couldn't send bgcolor to none opaque, if we want that functionality
	// need to draw 4 seperate quads
	CGfx::DrawQuad( V2( m_position.x, m_position.y ),
					V2( m_size.x, m_size.y),
					CSkinManager::GetColor( "dialog_box", "border_color", 0xFF00FF00));

	// Draw menu background (overlay over the menu border quad)
	CGfx::DrawQuad( V2( WINDOW_BORDER + m_position.x, WINDOW_BORDER + m_position.y),
					V2( m_size.x - WINDOW_BORDER * 2, m_size.y - WINDOW_BORDER * 2),
					CSkinManager::GetColor( "dialog_box", "background_color", 0xFF000000));

	// Draw the text
	V2 text_position(m_position.x + WINDOW_BORDER + WINDOW_PADDING, m_position.y + WINDOW_BORDER + WINDOW_PADDING);
	CFont::GetActiveUiFont()->Print(m_text, text_position, CSkinManager::GetColor( "dialog_box", "text_color", 0xFFFFFFFF));
	
	// Render the inputs
	m_pControlManager->RenderControl();
}
//******************************************************************************
//	
//******************************************************************************
void	CDialogBoxInput::Process()
{
	CWindow::Process();

	if (m_pOKButton->getClicked()) {
		// Only return OK if text is in field
		if (m_includeTextInput) {
			if (m_pTextField->getText()->size() != 0) {
				m_exitValue = OK;
			}
		} else {
			m_exitValue = OK;
		}
	}
	if (m_includeCancelButton) {
		if (m_pCancelButton->getClicked()) {
			m_exitValue = CANCEL;
		}
	}

	m_pControlManager->ProcessControl();
}

//******************************************************************************
//
//******************************************************************************
bool	CDialogBoxInput::ProcessInput()
{
	m_pControlManager->ProcessControlInput();
    return false;
}

//******************************************************************************
//
//******************************************************************************
int		CDialogBoxInput::getExitValue() {
	return m_exitValue;
}

CString	CDialogBoxInput::getReturnText() {
	if (m_includeTextInput) {
		return m_pTextField->getText()->toString();
	} else {
		return CString("");
	}
}
//*******************************  END OF FILE  ********************************
