/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) T Swann, Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CFileSystem.cpp

  Description :	

  Last Modified $Date: 2007/01/03 05:31:14 $

  $Revision: 1.2 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CFrameWork.h"
#include "CFileSystem.h"
#include "CConfigFile.h"

#include <vmime/platforms/psp/pspFile.hpp>

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************
static const s32	COPY_SIZE_IN_BYTES( 64 * 1024 );

SceIoDirent			CFileSystem::s_DirEntry;
CString				CFileSystem::s_szRootDirectory( "" );

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//******************************************************************************
//
//	File implenentation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CFile::CFile( const CString & filename, FILE * const p_handle )
:	m_szFilename( filename )
,	m_pHandle( p_handle )
,	m_Length( 0 )
{
	if ( Seek( 0, SEEK_END ) == true )
	{
		m_Length = Tell();

		if ( Seek( 0, SEEK_SET ) == false )
		{
			ASSERT( 0, "Failed to seek to beginning of file" );
		}
	}
	else
	{
		ASSERT( 0, "Failed to seek to the end of file" );
	}
}

//******************************************************************************
//	
//******************************************************************************
CFile::~CFile()
{
	fclose( m_pHandle );
}

//******************************************************************************
//	
//******************************************************************************
bool	CFile::Read( void * p_address, u32 length, u32 * p_nbytes_read ) const
{
	const u32	bytes_read( fread( p_address, 1, length, m_pHandle ) );

	if ( p_nbytes_read != NULL )
	{
		*p_nbytes_read = bytes_read;
	}

	return ( bytes_read == length );
}

//******************************************************************************
//	
//******************************************************************************
bool	CFile::Write( const void * p_address, u32 length, u32 * p_nbytes_written ) const
{
	const u32	bytes_written( fwrite( p_address, 1, length, m_pHandle ) );

	if ( p_nbytes_written != NULL )
	{
		*p_nbytes_written = bytes_written;
	}

	return ( bytes_written == length );
}

//******************************************************************************
//	
//******************************************************************************
bool	CFile::IsEOF() const
{
	return ( feof( m_pHandle ) != 0 );
}

//******************************************************************************
//	
//******************************************************************************
u32	CFile::GetLength() const
{
	return m_Length;
}

//******************************************************************************
//	
//******************************************************************************
u32	CFile::Tell() const
{
	return ftell( m_pHandle );
}

//******************************************************************************
//	
//******************************************************************************
bool	CFile::Seek( const u32 offset, const u32 origin ) const
{
	return ( fseek( m_pHandle, offset, origin ) == 0 );
}

//******************************************************************************
//	
//******************************************************************************
CString	CFile::GetExtension() const
{
	return CFileSystem::GetFileExtension( m_szFilename );
}

//******************************************************************************
//	
//******************************************************************************
char	CFile::FGetC() const
{
	char	character;

	if ( Read( &character, 1 ) == true )
	{
		return character;
	}

	return EOF;
}

//******************************************************************************
//	
//******************************************************************************
CFile::operator FILE * () const
{
	return m_pHandle;
}


//******************************************************************************
//******************************************************************************
//
//	File system implementation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::SetRoot( const CString & directory )
{
	if ( sceIoChdir( directory ) >= 0 )
	{
		vmime::platforms::psp::pspFileSystemFactory::setRoot(directory.GetStr());
		s_szRootDirectory = directory;

		return true;
	}

	return false;
}

//******************************************************************************
//	
//******************************************************************************
CFile *	CFileSystem::Open( const CString & filename, const char * const p_open_flags )
{
	FILE * const	p_handle( fopen( filename, p_open_flags ) );

	if ( p_handle == NULL )
	{
		return NULL;
	}

	return new CFile( filename, p_handle );
}

//******************************************************************************
//	
//******************************************************************************
void	CFileSystem::Close( CFile * const p_file )
{
	delete p_file;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::FileExists( const CString & filename )
{
	CFile * const	p_file( Open( filename, "rb" ) );

	if ( p_file == NULL )
	{
		return false;
	}

	Close( p_file );

	return true;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::DirectoryExists( const CString & directory )
{
	SceIoStat stat;

	if ( sceIoGetstat(directory.GetPtr(), &stat) < 0 )
	{
		return false;
	}

	return true;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::FindFirstFile( const CString & path, FIND_FILE_HANDLE & handle )
{
	handle = sceIoDopen( path );

	return ( handle >= 0 );
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::FindNextFile( sDirEntry & dir_entry, FIND_FILE_HANDLE handle )
{
	ASSERT( handle >= 0, "Cannot search with invalid directory handle" );

	const s32	error_code( sceIoDread( handle, &s_DirEntry ) );

	ASSERT( error_code >= 0, "Error reading directory entry" );

	if ( error_code > 0 )
	{
		dir_entry.m_Stats = s_DirEntry.d_stat;
		dir_entry.m_szFileName = s_DirEntry.d_name;

		return true;
	}

	return false;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::FindCloseFile( FIND_FILE_HANDLE handle )
{
	ASSERT( handle >= 0, "Trying to close an invalid directory handle" );

	return ( sceIoDclose( handle ) >= 0 );
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::GetDirectoryFiles( const CString & directory, CFileList & dir_files )
{
	bool				ret_code( false );
	FIND_FILE_HANDLE	handle;

	if ( FindFirstFile( directory, handle ) == true )
	{
		sDirEntry	dir_entry;

		ret_code = true;

		while ( FindNextFile( dir_entry, handle ) == true && ret_code == true )
		{
			if ( dir_entry.m_szFileName != "." && dir_entry.m_szFileName != ".." )
			{
				dir_entry.m_szFileName = CString( directory + CString( "/" ) + dir_entry.m_szFileName );

				dir_files.push_back( dir_entry );

				if ( dir_entry.IsDirectory() == true )
				{
					ret_code = GetDirectoryFiles( dir_entry.m_szFileName, dir_files );
				}
			}
		}

		FindCloseFile( handle );
	}
	else
	{
		ASSERT( 0, "FindFirstFile FAILED!" );
	}

	return ret_code;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::CopyFile( const CString & src_file, const CString & dst_file, float progress_inc )
{
	if ( FileExists( dst_file ) == true )
	{
		if ( DeleteFile( dst_file ) == false )
		{
			return false;
		}
	}

	CFile * const	p_src_file( Open( src_file, "rb" ) );
	CFile * const	p_dst_file( Open( dst_file, "wb" ) );

	if ( p_src_file != NULL && p_dst_file != NULL )
	{
		bool	success( true );
		s32		src_length( p_src_file->GetLength() );

		if ( src_length > 0 )
		{
			u8 *		p_buffer( new u8[ src_length ] );
			// BEN NOTE: These variable were unused!
			//const float	src_frac( ( src_length / COPY_SIZE_IN_BYTES ) + 1 );
			//const float	copy_frac_inc( src_frac == 0.f ? 0.f : ( progress_inc / src_frac ) );

			while ( src_length > COPY_SIZE_IN_BYTES && success == true )
			{
				if ( p_src_file->Read( p_buffer, COPY_SIZE_IN_BYTES ) == false )
				{
					success = false;
					ASSERT( 0, "CFileSystem::CopyFile() - FAILED!" );
				}

				if ( p_dst_file->Write( p_buffer, COPY_SIZE_IN_BYTES ) == false )
				{
					success = false;
					ASSERT( 0, "CFileSystem::CopyFile() - FAILED!" );
				}

				src_length -= COPY_SIZE_IN_BYTES;

			}

			if ( success == true )
			{
				if ( p_src_file->Read( p_buffer, src_length ) == false )
				{
					success = false;
					ASSERT( 0, "CFileSystem::CopyFile() - FAILED!" );
				}

				if ( p_dst_file->Write( p_buffer, src_length ) == false )
				{
					success = false;
					ASSERT( 0, "CFileSystem::CopyFile() - FAILED!" );
				}
			}

			SAFE_RDELETE( p_buffer );
		}

		Close( p_dst_file );
		Close( p_src_file );

		return success;
	}

	return false;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::CopyDirectory( const CString & src_dir, const CString & dst_dir, float progress_inc )
{
	//
	//	Gather the files in this directory
	//
	CFileList	dir_files;

	if ( GetDirectoryFiles( src_dir, dir_files ) == false )
	{
		ASSERT( 0, "GetDirectoryFiles FAILED!" );

		return false;
	}

	//
	//	Strip off the src path from the filenames
	//
	const u32	src_path_len( src_dir.Length() );

	for ( CFileList::iterator it = dir_files.begin(); it != dir_files.end(); ++it )
	{
		sDirEntry &	dir_entry( *it );

		dir_entry.m_szFileName = &dir_entry.m_szFileName[ src_path_len ];
	}

	//
	//	Make sure the destination folder exists
	//
	if ( MakeDirectory( dst_dir ) == false )
	{
		return false;
	}

	//
	//	Make sure all the destination directorys exist
	//
	for ( CFileList::iterator it = dir_files.begin(); it != dir_files.end(); ++it )
	{
		const sDirEntry &	dir_entry( *it );

		if ( dir_entry.IsDirectory() == true )
		{
			const CString	directory( dst_dir + dir_entry.m_szFileName );

			if ( MakeDirectory( directory ) == false )
			{
				return false;
			}
		}
	}

	//
	//	Copy all the files
	//
	for ( CFileList::iterator it = dir_files.begin(); it != dir_files.end(); ++it )
	{
		const sDirEntry &	dir_entry( *it );

		if ( dir_entry.IsFile() == true )
		{
			const CString	src_file( src_dir + dir_entry.m_szFileName );
			const CString	dst_file( dst_dir + dir_entry.m_szFileName );

			if ( CopyFile( src_file, dst_file, progress_inc ) == false )
			{
				return false;
			}
		}
	}

	return true;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::DeleteFile( const CString & filename )
{
	if ( MakeWritable( filename ) == true )
	{
		return ( sceIoRemove( filename ) >= 0 );
	}

	return false;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::DeleteDirectory( const CString & directory )
{
	//
	//	Get a list of all the files and folders to delete
	//
	CFileList	dir_files;

	if ( GetDirectoryFiles( directory, dir_files ) == false )
	{
		ASSERT( 0, "GetDirectoryFiles FAILED!" );

		return false;
	}

	//
	//	Delete all the files first
	//
	bool	no_files( false );

	while ( no_files == false && dir_files.empty() == false )
	{
		no_files = true;

		for ( CFileList::iterator it = dir_files.begin(); it != dir_files.end(); ++it )
		{
			const sDirEntry &	dir_entry( *it );

			if ( dir_entry.IsFile() == true )
			{
				if ( DeleteFile( dir_entry.m_szFileName ) == false )
				{
					return false;
				}

				no_files = false;

				dir_files.erase( it );

				break;
			}
		}
	}

	//
	//	Delete all the directorys last
	//
	bool	no_directory( false );

	while ( no_directory == false && dir_files.empty() == false )
	{
		no_directory = true;

		for ( CFileList::iterator it = dir_files.begin(); it != dir_files.end(); ++it )
		{
			const sDirEntry &	dir_entry( *it );

			if ( dir_entry.IsDirectory() == true )
			{
				no_directory = false;

				if ( sceIoRmdir( dir_entry.m_szFileName ) == 0 )
				{
					dir_files.erase( it );

					break;
				}
			}
		}
	}

	//
	//	Finally remove the parent directory
	//
	if ( sceIoRmdir( directory ) != 0 )
	{
		return false;
	}

	return true;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::MakeDirectory( const CString & directory )
{
	if ( DirectoryExists( directory ) == false )
	{
		return ( sceIoMkdir( directory, 0 ) == 0 );
	}

	return true;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::IsUMDInserted()
{
	const s32	ret_code( sceUmdCheckMedium( 0 ) );

	return ( ret_code > 0 );
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::ActivateUMD()
{
	if ( sceUmdWaitDriveStat( 2 ) < 0 )
	{
		return false;
	}

	if ( sceUmdActivate( 1, "disc0:" ) < 0 )
	{
		return false;
	}

	if ( sceUmdWaitDriveStat( UMD_WAITFORINIT ) < 0 )
	{
		return false;
	}

	return true;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::Rename( const CString & old_name, const CString & new_name )
{
	if ( MakeWritable( old_name ) == true )
	{
		return ( sceIoRename( old_name, new_name ) >= 0 );
	}

	return false;
}

//******************************************************************************
//	
//******************************************************************************
void	CFileSystem::SplitPath( const CString & path, CString * p_drive_out, CString * p_dir_out, CString * p_fname_out, CString * p_ext_out )
{
	int driveEnd, dirStart, dirEnd, extStart;
	
	//
	//	Find the extension
	//
	extStart = path.RFind(".");
	if (p_ext_out)
	{
		if (extStart >= 0)
			*p_ext_out = path.Substring(extStart+1);
	}
	
	//
	//	Find the fname
	//
	dirEnd = path.RFind("/");
	if (dirEnd < 0)
		dirEnd = path.RFind("\\");
	
	if (p_fname_out)
	{
		if (dirEnd >= 0 && extStart >= 0)
			*p_fname_out = path.Substring(dirEnd+1, extStart - dirEnd - 1);
		// No extensions mean we'll treat it as a dir
	}
	
	//
	//	Find the drive
	//
	driveEnd = path.Find(":");
	if (p_drive_out)
	{
		if (driveEnd >= 0)
			*p_drive_out = path.Substring(0, driveEnd + 1);
	}
	
	//
	//	What's left is the dir
	//
	if (p_dir_out)
	{
		dirStart = (driveEnd >= 0 ? driveEnd+1 : 0);
		if (dirEnd >= 0)
			*p_dir_out = path.Substring(dirStart, dirEnd - dirStart + 1);
		else
			*p_dir_out = path.Substring(dirStart);
	}
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::MakeReadOnly( const CString & filename )
{
	return false;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFileSystem::MakeWritable( const CString & filename )
{
	SceIoStat	stats;

	if ( sceIoGetstat( filename, &stats ) >= 0 )
	{
		const u32	bits( 1 );

		stats.st_mode &= FIO_SO_IFMT;
		stats.st_mode |= FIO_SO_IROTH | FIO_SO_IWOTH;

		if ( sceIoChstat( filename, &stats, bits ) >= 0 )
		{
			return true;
		}
		else
		{
			ASSERT( 0, "sceIoChstat FAILED!" );
		}
	}
	else
	{
		ASSERT( 0, "sceIoGetstat FAILED!" );
	}

	return false;
}

//******************************************************************************
//	
//******************************************************************************
CString	CFileSystem::MakeFullPath( const CString & filename )
{
	if ( filename.Find( s_szRootDirectory ) >= 0 )
	{
		return filename;
	}
	else
	{
		return ( s_szRootDirectory + filename );
	}
}

//******************************************************************************
//	
//******************************************************************************
CString	CFileSystem::GetFileExtension( const CString & filename )
{
	CString	ext;

	SplitPath( filename, NULL, NULL, NULL, &ext );

	return ext;
}

//******************************************************************************
//	
//******************************************************************************
s32 CFileSystem::DateTimeCompare(ScePspDateTime* date1, ScePspDateTime* date2)
{
	/* Returns 0 if equal,
	 *         > 0 if date1 after date2,
	 *         < 0 if date1 before date 2 */
	if (date1->year != date2->year)
		return (date1->year - date2->year);
	if (date1->month != date2->month)
		return (date1->month - date2->month);
	if (date1->day != date2->day)
		return (date1->day - date2->day);
	if (date1->hour != date2->hour)
		return (date1->hour - date2->hour);
	if (date1->minute != date2->minute)
		return (date1->minute - date2->minute);
	if (date1->second != date2->second)
		return (date1->second - date2->second);
	if (date1->microsecond != date2->microsecond)
		return (date1->microsecond - date2->microsecond);
	return 0;
}

//******************************************************************************
//	
//******************************************************************************
s32 CFileSystem::GetModificationTime( const CString & filename, ScePspDateTime* dateTime )
{
	s32 err;
	SceIoStat stat;

	if ( (err = sceIoGetstat(filename.GetPtr(), &stat)) < 0 )
		return err;
	
	memcpy(dateTime, &stat.st_mtime, sizeof(ScePspDateTime));
	return 0;
}

//*******************************  END OF FILE  ********************************
