/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) T Swann, Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CFrameWork.cpp

  Description :	

  Last Modified $Date: 2007/02/10 03:03:53 $

  $Revision: 1.10 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CFrameWork.h"
#include "CPSPoste.h"
#include "CGfx.h"
#include "CUSBManager.h"
#include "CInput.h"
#include "CFileSystem.h"
#include "CProcess.h"
#include "CFont.h"
#include "CSkinManager.h"
#include "CExecutableFileHandler.h"
#include "CFileHandler.h"
#include "CBackground.h"
#include "CRenderable.h"
#include "CWindow.h"
#include "CConfigFile.h"
#include "CContacts.h"
#include "CAccountTemplatesFile.h"
#include "CConnectionManager.h"
#include "CDialogModal.h"
#include <time.h>

#include <vmime/vmime.hpp>
#include <vmime/platforms/psp/pspHandler.hpp>

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************
u64					CFrameWork::s_Time;
u64					CFrameWork::s_LastTime;
u64					CFrameWork::s_TickResolution;
CFile *				CFrameWork::s_pLogFile;

extern "C" {
// This flag must not be mangled 'cause it's imported in CAssert.cpp to catch
// home button presses when in an assert.
bool				s_ExitCallbackCalled( false );
}

static u32			s_nVersion;
static CString		s_szMarkerFile( "www.psposte.org" );
static CString		s_szUsb("on");

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
// Set the application to exit
//******************************************************************************
void CFrameWork::setExit() {
	s_ExitCallbackCalled = true;
}


//******************************************************************************
//	Framework exit point
//******************************************************************************
static int	ExitCallback( int arg1, int arg2, void * common )
{
	s_ExitCallbackCalled = true;

	return 0;
}

//******************************************************************************
//	Power event callback
//******************************************************************************
int PowerCallback(int unknown, int pwrflags, void *common)
{
	// Note that the auto suspend doesn't seem to generate a callback so
	// instead, we call scePowerTick() every frame
	if (pwrflags & PSP_POWER_CB_POWER_SWITCH || pwrflags & PSP_POWER_CB_SUSPENDING) {
		CLog::Close();
	} else if (pwrflags & PSP_POWER_CB_RESUMING) {
		
	} else if (pwrflags & PSP_POWER_CB_RESUME_COMPLETE) {
		CLog::Open();
	} else if (pwrflags & PSP_POWER_CB_STANDBY) {
		CLog::Close();
	} else {
		CLog::Debug("first arg: 0x%08X, flags: 0x%08X: Unhandled power event\n",
		            unknown, pwrflags);
	}
	
	sceDisplayWaitVblankStart();

	return 0;
}

//******************************************************************************
//	HOME button & Power event callback thread
//******************************************************************************
static int	CallbackThread( SceSize args, void * argp )
{
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", ExitCallback, NULL);
	sceKernelRegisterExitCallback(cbid);
	cbid = sceKernelCreateCallback("Power Callback", PowerCallback, NULL);
	scePowerRegisterCallback(0, cbid);
	
	sceKernelSleepThreadCB();

	return 0;
}
//******************************************************************************
//	Setup the HOME button callback & Power event callback
//******************************************************************************
int	SetupCallbacks()
{
	const int thid( sceKernelCreateThread( "update_thread", CallbackThread, 0x11, 0xFA0, 0, 0 ) );

	if ( thid >= 0 )
	{
		sceKernelStartThread( thid, 0, 0 );
	}
	else
	{
		CLog::Error("Failed to create the home & power callback thread: 0x%x\n", thid);
	}

	return thid;
}

//******************************************************************************
//	
//******************************************************************************
void setKernelPatch()
{
	//Patch PSP OS
	s_nVersion = *( ( u32 * )0x8805c510 );		// Save the real version number

	if ( CFrameWork::GetVersion() == CFrameWork::PSP_v1_5 )
	{
		// Firmware v1.5
		*((unsigned int *)0x8805c510)=0x00001021; // good PRX
		*((unsigned int *)0x8805c834)=0x00000000; // good VSH
		*((unsigned int *)0x8805c690)=0x00000000; // VSHByID
		*((unsigned int *)0x8805c3b8)=0x00000000; // modByID
		*((unsigned int *)0x8805c834)=0x00000000; // VSH
		*((unsigned int *)0x8805ca40)=0x00000000; // VSHPlain

		//that will write a j MyFunction to AddressOfFunction
		//u32 data;
		//data = (u32) MyFunction & 0xFFFFFFF;
		//data = 0x8000000 | (data >> 2);
		//_sw(data, AddressOfFunction);

		//Patch sceExitGame
		//*((unsigned int *)0x8806882c)=0x03e00008; // disable sceExitGame with jr $ra
		//*((unsigned int *)0x88068830)=0x00001021; // addu v0, zero, zero
#ifdef DEBUG
		printf( "\tKernel patched : \n\t\tPRX loadmodule,\n\t\tsceExitGame\n" );
		sceKernelDelayThread(1000000);
#endif
	}
	else
	{
#ifdef DEBUG
		printf( "Bad OS version, 1.5 required, no patch\n" );
		sceKernelDelayThread(1000000);
#endif
	}
}

//******************************************************************************
//	Initialise the framework sub systems
//******************************************************************************
bool	CFrameWork::Open()
{
	//
	//	Record the start time
	//
	s_TickResolution = sceRtcGetTickResolution();
	sceRtcGetCurrentTick( &s_Time );
	s_LastTime = s_Time;
	
	//
	//	Set the timezone
	//
	tzset();
	
	//
	//	Disable the Idle timer (prevents corrupting the MS)
	//
	scePowerIdleTimerDisable(0);
	
	//
	//	Setup our HOME button callbacks
	//
	SetupCallbacks();
	
	//
	//	Set the clock speed
	//
//	scePowerSetBusClockFrequency( 166 );
//	scePowerSetCpuClockFrequency( 333 );
	
	//
	//	Open the FileAssistant log file
	//
	s_pLogFile = CFileSystem::Open( s_szMarkerFile, "wb" );

	ASSERT( s_pLogFile != NULL, "Failed to open log file" );

	//
	//	Initialize the VMime platform-specific handler & init TLS/SSL
	//
	vmime::platform::setHandler <vmime::platforms::psp::pspHandler>();
	vmime::net::tls::TLSSession::oneTimeInit();

	//
	//	Initialise the random seed
	//
	srand( static_cast< u32 >( time( NULL ) ) );

	//
	//	Open the configuration file
	//
	if (CConfigFile::Open() == false) {
		ASSERT(0, "Failed to load from settings");

		return false;
	}

	//
	//  Open the contacts file
	//
	if (CContacts::Open() == false) {
		ASSERT(0, "Failed to load from contacts");

		return false;
	}

	// 
	// Open the account templates file
	//
	if (CAccountTemplatesFile::Open() == false) {
		ASSERT(0, "Failed to load from account templates file");
		
		return false;
	}

	//
	//	Open the process manager
	//
	CProcess::Open();

// Use of USB deprecated: it caused corruption and psplink should be used for
// development.
#if 0
	//
	//	Initialise the USB manager
	//
	if ( CUSBManager::Open() == false )
	{
		ASSERT( 0, "USB Manager failed to open" );

		return false;
	}
#endif	// #ifdef _DEBUG

	//
	//	Initialise the graphics sub system
	//
	if( CGfx::Open() == false )
	{
		ASSERT( 0, "GPU failed to initialise" );

		return false;
	}

	//
	//	Open the render manager
	//
	CRenderable::Open();

	//
	//	Open the skin manager
	//
	CSkinManager::Open();

	//
	//	Open the font system
	//
	CFont::Open();

	//
	//	Initialise the input controller
	//
	if ( CInput::Open() == false )
	{
		ASSERT( 0, "Input Manager failed to open" );

		return false;
	}

	//
	//	Open the file handler class
	//
	if ( CFileHandler::Open() == false )
	{
		ASSERT( 0, "File handler failed to open" );

		return false;
	}

	//
	//	Open the background renderer
	//
	//CBackground::Open();

	//
	//	Open the window manager
	//
	CWindow::Open();
	
	//
	//	Open the modal dialog manager
	//
	CDialogModal::Open();

// Use of USB deprecated: it caused corruption and psplink should be used for
// development.
#if 0
	//
	//	In debug we always want the USB connection to be on
	//
	if ( s_szUsb == "on" )
	{
		if ( CUSBManager::Activate() == false )
		{
			ASSERT( 0, "Failed to activate USB connection" );
		}
	}
#endif
	
	return true;
}

//******************************************************************************
//	
//******************************************************************************
void	CFrameWork::Close()
{
// Use of USB deprecated: it caused corruption and psplink should be used for
// development.
#if 0
	//
	//	Shut down the USB manager
	//
	CUSBManager::Close();
#endif	// #ifdef _DEBUG

	//
	//	Close the modal dialog manager
	//
	CDialogModal::Close();

	//
	//	Close the window manager
	//
	CWindow::Close();

	//
	//	Close the background renderer
	//
	//CBackground::Close();
	
	//
	//	Close the file handler class
	//
	CFileHandler::Close();

	//
	//	Close the input controller
	//
	CInput::Close();

	//
	//	Close the skin manager
	//
	CSkinManager::Close();

	//
	//	Close the font system
	//
	CFont::Close();

	//
	//	Close the render manager
	//
	CRenderable::Close();

	//
	//	Shut down the GPU
	//
	CGfx::Close();

	//
	//	Close the config file
	//
	CConfigFile::Close();

	//
	// Close the contacts file
	//
	CContacts::Close();

	//
	// Close the account templates file
	//
	CAccountTemplatesFile::Close();

	//
	// De-init the TLS/SSL libs
	//
	vmime::net::tls::TLSSession::oneTimeDeInit();

	//
	//	Close the FileAssistant log file
	//
	CFileSystem::Close( s_pLogFile );
	
	//
	//	Close the process manager
	//
	CProcess::Close();

	//
	//	Close the PSPoste log files
	//
	CLog::Close();
}

//******************************************************************************
//	
//******************************************************************************
int CFrameWork::Process()
{
	//
	//	Update the timers
	//
	s_LastTime = s_Time;
	sceRtcGetCurrentTick( &s_Time );

	//
	//	Update the input
	//
	CInput::Process();

	//
	//	Update the process list
	//
	CProcess::ProcessList();
	
	//
	//	Update the modal dialogs
	//
	CDialogModal::Process();

	if ( s_ExitCallbackCalled == true )
	{
		CPSPoste::Destroy();
		CFrameWork::Close();
		sceKernelExitGame();
		return 0;
		//ExitCallback( 0, 0, NULL ); //sceKernelExitGame();
	}

	//
	//	Render everything
	//
	CRenderable::Render();

	return 0;
}

//******************************************************************************
//	
//******************************************************************************
void	CFrameWork::DBGMessage( const char * fmt, ... )
{
	pspDebugScreenPrintf( fmt );
}

//******************************************************************************
//	
//******************************************************************************
u32		CFrameWork::GetTicks()
{
	return s_Time;
}

//******************************************************************************
//	
//******************************************************************************
u32		CFrameWork::GetTicksMs()
{
	return u32(s_Time)/1000;
}

//******************************************************************************
//	
//******************************************************************************
float	CFrameWork::GetElapsedTime()
{
	return ( static_cast< float >( s_Time - s_LastTime ) / s_TickResolution );
}

//******************************************************************************
//	
//******************************************************************************
SceUID	CFrameWork::LoadModule( const CString & module, eModulePartition partition )
{
	SceUID	load_id;

	switch ( partition )
	{
	case KERNEL_PARTITION:
		{
			load_id = sceKernelLoadModule( module, 0, NULL );
		}
		break;

	case USER_PARTITION:
		{
			SceKernelLMOption	option;

			option.flags = 0;
			option.access = 1;
			option.position = 0;
			option.mpidtext = 2;
			option.mpiddata = 2;
			option.size = sizeof( option );

			load_id = sceKernelLoadModule( module, option.flags, &option );
		}
		break;
	}

#ifdef _DEBUG
	if ( load_id & 0x80000000 )
	{
		BREAK_POINT( "sceKernelLoadModule - %X", load_id );
	}
#endif // _DEBUG

	return load_id;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFrameWork::LoadAndStartModule( const CString & module, eModulePartition partition )
{
	s32		status;
	SceUID	start_id;
	SceUID	load_id( LoadModule( module, partition ) );

	if ( ( load_id & 0x80000000 ) == 0 )
	{
		start_id = sceKernelStartModule( load_id, 0, NULL, &status, NULL );
	}

	if ( load_id != start_id )
	{
		BREAK_POINT( "sceKernelStartModule - %X, %X", start_id, status );
		return false;
	}

	return true;
}

//******************************************************************************
//	
//******************************************************************************
bool	CFrameWork::RunExecutable( const CString & executable )
{
	SceKernelLoadExecParam	execParam;
	const u32				total_length( executable.Length() + 1 );

	execParam.args = total_length;
	execParam.argp = const_cast< char * >( executable.GetPtr() );
	execParam.key = NULL;
	execParam.size = sizeof( execParam ) + total_length;

	if ( sceKernelLoadExec( executable, &execParam ) < 0 )
	{
		return false;
	}

	return true;
}

//******************************************************************************
//	
//******************************************************************************
CFrameWork::eFirmwareVersion	CFrameWork::GetVersion()
{
	if ( s_nVersion == 0x0e017c43 )
	{
		return PSP_v1_5;
	}

	return PSP_v1_0;
}

//*******************************  END OF FILE  ********************************
