/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CKeyboard.h

  Description :	

  Last Modified $Date: 2006/11/22 02:37:40 $

  $Revision: 1.3 $

*******************************************************************************/

#ifndef CKEYBOARD_H_
#define CKEYBOARD_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CTypes.h"

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Types
//******************************************************************************

//******************************************************************************
//   Constants
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
/* The CKeyboard class defines the interface PSPoste is to use when wanting to
 * poll for key presses when in text entry mode. See the CKeyboardNav class for
 * the class responsible for navigating the interface.
 */
class CKeyboard
{
	public: /* Types */
	
		enum eKeyboardRender
		{
			eKeyboardRender_Enabled,
			eKeyboardRender_EnabledForFrame,
			eKeyboardRender_Disabled,
			eKeyboardReander_Max
		};
		
		enum eKeyboardState
		{
			eKeyboardState_Active,
			eKeyboardState_Inactive,
			eKeyboardState_Max
		};
		
		enum eKeyboardKeyPressed
		{
			eKeyboardKeyPressed_None = 0,
			eKeyboardKeyPressed_Char,
			eKeyboardKeyPressed_Control
		};
		
		enum eKeyboardControlKey
		{
			UP = 1,
			DOWN,
			LEFT,
			RIGHT,
			PAGE_UP,
			PAGE_DOWN,
			BACKSPACE,
			DELETE,
			ESC,
			ENTER,
			HOME,
			END,
			TAB
		};
	
	public: /* Constructors & Destructors */
	
		virtual ~CKeyboard() {};
		CKeyboard() : 
			m_charKey(0),
			m_controlKey(0),
			m_renderEnabled(false),
			m_inputEnabled(false),
			m_shouldDisable(false)
		{};
	
	public: /* Methods relating to rendering the keyboard */
	
		/* Returns true if the keyboard will render the next time Render is called */
		virtual bool isRenderEnabled() { return m_renderEnabled; };
		
		/* Enables rendering of the keyboard until disableRender() is called */
		virtual void enableRender() { m_renderEnabled = true; };
		
		/* Disables rendering of the keyboard */
		virtual void disableRender() { m_renderEnabled = false; };
		
		/* Renders the keyboard if the current render state allows, returns true if rendering occurred */
		virtual bool render()
		{
			if (m_renderEnabled)
				return doRender();
			return false;
		};
		
	public: /* Methods relating to polling the keyboard for "key presses" */
	
		/* Returns true if the keyboard Process will actually do something */
		virtual bool isInputEnabled() { return m_inputEnabled; };
	
		/* Will enable polling of the keyboard */
		virtual void enableInput() { m_inputEnabled = true; };
		
		/* Will disable polling of the keyboard */
		virtual void disableInput() { m_inputEnabled = false; m_shouldDisable = false; };
		
		/* Polls the keyboard if it is active. Returns true if the keyboard was left in the enabled state. */
		virtual bool process(SceCtrlData *aPadData)
		{
			if (m_inputEnabled)
				return doProcess(aPadData);
			return false;
		};
		
		/* Intended for users to know when a "disable" key sequence has been entered on the keyboard so the
		 * application can know to return to navigation mode. */
		virtual bool shouldDisable()
		{
			return m_shouldDisable;
		};
		
		/* Returns whether a char or control key was pressed if the keyboard is active and the last Process
		 * produced a key. Returns eKeyboardKeyPressed_None if the keyboard wasn't active or the Process didn't
		 * produce a key */
		virtual eKeyboardKeyPressed isKeyboardPressed()
		{
			if (m_inputEnabled)
			{
				if (m_charKey)
					return eKeyboardKeyPressed_Char;
				else if (m_controlKey)
					return eKeyboardKeyPressed_Control;
			}
			
			return eKeyboardKeyPressed_None;
		};
		
		/* Returns the character key produced by the last Process, note that 0 means no char key was produced.
		 * Note that a control key may have been produced though! Character keys are defined as printable
		 * characters + the space, tab, carriege return and newline keys. */
		virtual u16 getCharKey() { return m_charKey; };
		
		/* Returns the control key produced by the last Process, note that 0 means no control key was produced.
		 * Note that a character key may have been produced though!
		 * 
		 * Control keys are standardized accross all keyboards, so a specific implementation may have to
		 * remap its control keys to the values specified in this class. */
		virtual u16 getControlKey() { return m_controlKey; };
		
		/* Returns true if the HUH should be allowed to render. */
		virtual bool allowHUH() { return !isRenderEnabled(); };
		
	protected: /* Methods to be overridden to actually do something */
	
		virtual bool doRender() { return false; };
		virtual bool doProcess(SceCtrlData *aPadData) { return false; };
		
	public:
	
		virtual void reset() {};

	protected:
	
		u16 m_charKey;
		u16 m_controlKey;
		
		bool m_renderEnabled;
		bool m_inputEnabled;
		bool m_shouldDisable;
};

//******************************************************************************
//   Externs
//******************************************************************************

//******************************************************************************
//   Prototypes
//******************************************************************************

#endif /* CKEYBOARD_H_ */
