/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CMailbox.h

  Description :	

  Last Modified $Date: 2006/12/15 14:38:09 $

  $Revision: 1.7 $

*******************************************************************************/

#ifndef CMAILBOX_H_
#define CMAILBOX_H_

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CString.h"
#include "CTypes.h"
#include "CConfigFile.h"
#include "CProcess.h"
#include "CCertRequestHandler.h"
#include "CCertificateManager.h"
#include <pspthreadman.h>
#include <time.h>
#include <vmime/vmime.hpp>

//******************************************************************************
//   Macros
//******************************************************************************

//******************************************************************************
//   Class definitions
//******************************************************************************
class MMailboxEventListener;
class CMaildirStore;
class CMailRetriever;
class CMailSender;
class CMessageHeader;
class CSendMsg;
class MMailboxSort;
class CCertRequest;
class CCertRequestHandler;

//******************************************************************************
//   Types
//******************************************************************************
enum EMailboxMsgType
{
	EMailboxMsgType_RetrievalStarted,
	EMailboxMsgType_RetrievedOne,
	EMailboxMsgType_RetrievalDone, // Done and retrieved at least 1
	EMailboxMsgType_RetrievedNone, // Done and retrieved none
	EMailboxMsgType_RetrieveFailed,
	EMailboxMsgType_RetrieveDisconnected, // From the mail server
	EMailboxMsgType_SentOne,
	EMailboxMsgType_SendDone,
	EMailboxMsgType_SendFailed,
	EMailboxMsgType_SendDisconnected, // From the mail server
	EMailboxMsgType_StoringStarted,
	EMailboxMsgType_StoredOne,
	EMailboxMsgType_StoringDone,
	EMailboxMsgType_StoredNone,
	EMailboxMsgType_StoreFailed,
	EMailboxMsgType_CertRequest
};

class SMailboxMsg : public SCommMsg
{
	public:
		SMailboxMsg(u32 aType) : SCommMsg(aType) {};
	
	public:
		int index; // RetrievedOne, SentOne uses this field
		int count; // RetrievedStarted, RetrievedOne, RetrievedDone, SentOne, SendDone use this field
		std::vector<CSendMsg*>* sentList; // SendDone, SendFailed use this field
		std::vector<CSendMsg*>* failedList; // SendFailed use this field
	
		CString errorText; // Used by RetrieveFailed, SendFailed, StoreFailed
	
		CCertRequest* certRequest; // Used by CertRequest
};

//******************************************************************************
//   Constants
//******************************************************************************
#define SEND_ERROR_NO_MESSAGES -1
#define SEND_ERROR_ALREADY_SENDING -2

bool messageHeaderBinaryPredicate(CMessageHeader* aHeader1,
                                  CMessageHeader* aHeader2);

//******************************************************************************
//******************************************************************************
//
//	Messagebox class
//
//******************************************************************************
//******************************************************************************
class CMailbox : public CProcess
{
	public: // Typedefs
	
		typedef std::vector<CMessageHeader*> TMessageList;
		typedef std::vector<CSendMsg*> TSendMsgList;

	public: // Constructors & Destructors
	
		CMailbox(CAccount* account);
		~CMailbox();
		
	public: // Inherited methods
	
		void	Process();
		bool	ProcessInput();
		
	public: // New methods
	
		void	subscribe(MMailboxEventListener* aListener);
		void	unsubscribe(MMailboxEventListener* aListener);
	
		int 	deleteMessage(CMessageHeader* aMsg);
		int 	deleteMarkedMessages();
	
		int		retrieveNewMessages();
		void	cancelRetrieve();
		void	requestBodyPart(u32 aId); // Will contact the server
		void	getBodyPart(u32 aId); // Will get from local store
	
		int 	getUnreadMessageCount();
		int		getReadMessageCount();
		int 	getTotalMessageCount();
		bool	isRetrieving();
		
		// Return -ive if aHeader1 < aHeader2, 0 if ==, +ve if >
		int		compareMessageHeaders(const CMessageHeader* aHeader1,
		                              const CMessageHeader* aHeader2) const;

		CString getBody(CMessageHeader* aMsg);
		void markRead(CMessageHeader* aMsg, bool aIsRead);

		void setSort(MMailboxSort *aSort);
		
	public: // Deprecated!!
	
		void 	setUnreadMessageCount(int count);

	private: // New methods

		void clearMsgList();	
		void clearSendMsgList(TSendMsgList* aSendMsgList = NULL);
		CString getMaildirRelativePath();
		CString getMaildirAbsolutePath();
		void insertNewMessages();
		void bisectionInsert(CMessageHeader* aHeader);
		
		void notifyEvent(SMailboxMsg* aEvent);

	/* BEGIN SEND */
	private: 
		TSendMsgList* m_pSendMsgList;
		TSendMsgList* m_pReadyToSend;

	public:
		void addSendMsg(CSendMsg* msg);
		int sendAllMessages();
		bool isSending();

	private:
		void SendTest();

	private:
		// Has a request been made to send mail?
		bool m_send_mail;

		bool m_sending;
	/* END SEND */


	public: // Saj did this
	
		TMessageList *      m_pMessageList;
		std::list<MMailboxEventListener*> m_mailbox_listeners;

	private: // Data members
	
		int                 m_unreadMessageCount;
		CAccount*           m_account;
		
		// Mailboxes used to communicate with the retriever and sender threads
		SceUID m_mailbox_mbx;
		SceUID m_retriever_mbx;
		SceUID m_sender_mbx;
		
		// The threads
		SceUID m_retriever_thread;
		SceUID m_sender_thread;

		// The retriever, sender and store
		CMailRetriever* m_retriever;
		CMailSender* m_sender;
		CMaildirStore* m_store;
		
		// The object used to sort the message list
		MMailboxSort* m_sort;
		
		// Has a request been made to check for new mail?
		bool m_check_mail;
		
		// This is used to prevent infinite connection attempts
		bool m_waiting_for_con_man;
		
		bool m_retrieving;
		
		int m_last_retrieved_num;

		vmime::ref<CCertificateManager> m_certManager;
		CCertRequestHandler* m_certRequestHandler;
};

class MMailboxEventListener
{
	public: // Constructors & Destructors
	
		virtual ~MMailboxEventListener() {};
	
	public: // New methods
	
		virtual void retrievalStarted(CMailbox* aMailbox, u32 aTotal) {};
		virtual void retrievedOne(CMailbox* aMailbox, u32 aIndex, u32 aTotal) {};
		virtual void retrievalDone(CMailbox* aMailbox, u32 aTotal) {};
		virtual void retrievedNone(CMailbox* aMailbox) {};
		virtual void retrieveFailed(CMailbox* aMailbox, CString aErrorText) {};
		
		virtual void sentOne(CMailbox* aMailbox, u32 aIndex, u32 aTotal) {};
		virtual void sendDone(CMailbox* aMailbox, u32 aTotal) {};
		virtual void sendFailed(CMailbox* aMailbox, CString aErrorText) {};

		virtual void storingStarted(CMailbox* aMailbox, u32 aTotal) {};
		virtual void storedOne(CMailbox* aMailbox, u32 aIndex, u32 aTotal) {};
		virtual void storingDone(CMailbox* aMailbox, u32 aTotal) {};
		virtual void storedNone(CMailbox* aMailbox) {};
		virtual void storeFailed(CMailbox* aMailbox, CString aErrorText) {};

};

class CMessageHeader
{
	public:

		// The constructor takes in a mailbox so it can be used for sorting
		// according to the mailbox settings
		CMessageHeader(CMailbox* aMailbox) : m_mailbox(aMailbox) {};
		vmime::ref<const vmime::addressList> to;
		vmime::ref<const vmime::mailbox> from;
		vmime::ref<const vmime::addressList> cc;
		vmime::ref<const vmime::addressList> bcc;
		time_t date;
		CString subject;
		int num;
		int	forwarded;
		int replied;
		bool unread;
		bool marked;
		
		CMailbox* m_mailbox; // Required for sorting
		
};

class MMailboxSort
{
	public:
		virtual int compare(const CMessageHeader* aHeader1, 
		                    const CMessageHeader* aHeader2) const = 0;

		virtual ~MMailboxSort() {};
};

class CMailboxSortDate : public MMailboxSort
{
	public:
		int compare(const CMessageHeader* aHeader1, 
		            const CMessageHeader* aHeader2) const
		{
			if (aHeader1->date < aHeader2->date)
				return -1;
			else if (aHeader1->date > aHeader2->date)
				return 1;
			return 0;
		}
};

class CMailboxSortNum : public MMailboxSort
{
	public:
		int compare(const CMessageHeader* aHeader1, 
		            const CMessageHeader* aHeader2) const
		{
			return (aHeader1->num - aHeader2->num);
		}
};

class CMailboxSortFrom : public MMailboxSort
{
	public:
		int compare(const CMessageHeader* aHeader1, 
		            const CMessageHeader* aHeader2) const
		{
			return aHeader1->from->getEmail().compare(aHeader2->from->getEmail());
		}
};

class CMailboxSortSubject : public MMailboxSort
{
	public:
		int compare(const CMessageHeader* aHeader1, 
		            const CMessageHeader* aHeader2) const
		{
			return strcasecmp(aHeader1->subject.GetPtr(), aHeader2->subject.GetPtr());
		}
};

// This class is a decorator for desc sorts
class CMailboxDescSort : public MMailboxSort
{
	public:
		CMailboxDescSort(MMailboxSort* aSort) : m_sort(aSort) {};
		int compare(const CMessageHeader* aHeader1, 
		            const CMessageHeader* aHeader2) const
		{
			return -(m_sort->compare(aHeader1, aHeader2));
		}
		
		~CMailboxDescSort() {SAFE_DELETE(m_sort);};
		
	private:
		MMailboxSort* m_sort;
};

class CSendMsg
{
	public:
		CSendMsg() {};

	public:
		vmime::addressList to;
		vmime::addressList cc;
		vmime::addressList bcc;
		vmime::string subject;
		vmime::string body;
};


#endif /* CMAILBOX_H_ */
