/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CMaildirStore.cpp

  Description :	

*******************************************************************************/

#include "CMaildirStore.h"
#include "CFileSystem.h"
#include "CLog.h"

#include <vmime/utility/stream.hpp>

#include <iostream>
#include <sstream>
#include <fstream>
#include <algorithm>

CMaildirStore::CMaildirStore(const CString& aPath, CMailbox* aMailbox, SceUID aMailboxMbx) :
	m_mailbox_mbx(aMailboxMbx),
	m_path(aPath),
	m_mailbox(aMailbox),
	m_store_ref(NULL),
	m_folder_ref(NULL)
{
	CString maildirUrl("maildir://localhost");
	maildirUrl += m_path;
	
	try
	{
		VUrl url(maildirUrl);

		// Create a new session
		m_sess_ref = vmime::create <VSession>();
		
		// Create an instance of the store
		m_store_ref = m_sess_ref->getStore(url);
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
	}
	
	restoreUids(); // Read the UIDs from file

	// Create the binary semaphore
	m_sema = sceKernelCreateSema(maildirUrl + "_Sema", 0, 1, 1, NULL);
}

CMaildirStore::~CMaildirStore()
{
	try
	{
		connect();
		open(V_MODE_READ_WRITE);
	}
	catch (std::exception& e)
	{
		// Don't throw from a destructor!
	}
	
	try
	{
		if (m_folder_ref && m_folder_ref->isOpen())
			m_folder_ref->expunge();

		close();
		disconnect();
	}
	catch (std::exception& e)
	{
		// Don't throw from a destructor!
	}
	
	saveUids(); // Write the uids to file
	
	sceKernelDeleteSema(m_sema);
}

void CMaildirStore::getMessages(CMailbox::TMessageList* aMsgList, bool sort, int from, int to)
{
	// The lock/unlock must happen outside the try/catch!!
	lock();
	
	try
	{
		connect();
		open(V_MODE_READ_ONLY);
		sysGetMessages(aMsgList, sort, from, to);
	}
	catch (vmime::exceptions::message_not_found& e)
	{
		// No big deal.
		// Sometimes means there's no messages on the server, other times that
		// there's no message in the store.
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
	}
	
	try
	{
		close();
		disconnect();
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
	}
	
	// The lock/unlock must happen outside the try/catch!!
	unlock();
}

u32 CMaildirStore::getMessageCount()
{
	u32 toRet = 0;
	
	// The lock/unlock must happen outside the try/catch!!
	lock();
	
	try
	{
		connect();
		open(V_MODE_READ_ONLY);
		toRet = sysGetMessageCount();
	}
	catch (vmime::exception& e)
	{
		toRet = 0;
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		toRet = 0;
		S_EXCEPT(e);
	}
	
	try
	{
		close();
		disconnect();
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
	}
	
	// The lock/unlock must happen outside the try/catch!!
	unlock();
	
	return toRet;
}

void CMaildirStore::addMessages(VMessageRefList aMsgs)
{
	// The lock/unlock must happen outside the try/catch!!
	lock();
	
	try
	{
		connect();
		open(V_MODE_READ_WRITE);
		sysAddMessages(aMsgs);
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
		failed(e.what());
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
		failed(e.what());
	}
	
	try
	{
		close();
		disconnect();
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
	}
	
	// The lock/unlock must happen outside the try/catch!!
	unlock();
}

void CMaildirStore::connect()
{
	if (!m_store_ref)
		throw vmime::exception("Store was uninitialized");
		
	if (!m_store_ref->isConnected())
		m_store_ref->connect();
}

void CMaildirStore::disconnect()
{
	if (!m_store_ref)
		throw vmime::exception("Store was uninitialized");
		
	if (m_store_ref->isConnected())
		m_store_ref->disconnect();
}

void CMaildirStore::open(const int aMode)
{
	if (m_folder_ref && m_folder_ref->isOpen())
		return;
		
	m_folder_ref = m_store_ref->getDefaultFolder();
	if (!m_folder_ref->exists())
		m_folder_ref->create(vmime::net::folder::TYPE_CONTAINS_MESSAGES);
	
	if (!m_folder_ref->isOpen())
		m_folder_ref->open(aMode);
}

void CMaildirStore::close()
{
	if (m_folder_ref && m_folder_ref->isOpen())
	{
		m_folder_ref->close(false);
		m_folder_ref = NULL;
	}
}

void CMaildirStore::lock()
{
	 sceKernelWaitSema(m_sema, 1, NULL);
}

void CMaildirStore::unlock()
{
	sceKernelSignalSema(m_sema, 1);
}

void CMaildirStore::sysGetMessages(CMailbox::TMessageList* aMsgList, bool sort, int from, int to)
{
	// Will retrieve the desired messages from the store
	VMessageRefList msgList = m_folder_ref->getMessages(from, to);
	VMessageRefList::iterator it = msgList.begin();
	CMessageHeader* msgHeader;
	
	while (it != msgList.end())
	{
		msgHeader = getMessageHeader(*it);
		aMsgList->push_back(msgHeader);
		it++;
	}
	
	if (sort)
		sortMessageList(aMsgList);
}

u32 CMaildirStore::sysGetMessageCount()
{
	if (m_folder_ref)
		return m_folder_ref->getMessageCount();
	return 0;
}

void CMaildirStore::sysAddMessages(VMessageRefList aMsgs)
{
	// Add them to the maildir store
	VMessageRefList::iterator it = aMsgs.begin();
	int cur = 0;
	int tot = aMsgs.size();
	start(tot);
	
	while (it != aMsgs.end())
	{
		sysAddMessage(*it);
		progress(++cur, tot);
		it++;
	}
	
	stop(tot);
}

void CMaildirStore::sysAddMessage(VMessageRef aMsg)
{
	std::ostringstream oss;
	vmime::utility::outputStreamAdapter osAdapter(oss);
	
	aMsg->extract(osAdapter);

	const std::string& str = oss.str();
	vmime::utility::inputStreamStringAdapter issAdapter(str);
	
	m_folder_ref->addMessage(issAdapter, str.length());
	
	// VMime will return a uid with \r at the end for every message but the last
	m_uids.push_back(vmime::utility::stringUtils::trim(aMsg->getUniqueId()));
}

void CMaildirStore::saveUids()
{
	std::ofstream ofs;
	std::vector<std::string>::iterator it = m_uids.begin();
	
	ofs.open(CFileSystem::MakeFullPath(m_path) + "uids");
	if (ofs.fail())
	{
		CLog::Error("Error openning the uids file: %suids", CFileSystem::MakeFullPath(m_path).GetPtr());
		ofs.close();
		return;
	}
	
	while (it != m_uids.end())
	{
		ofs << *it << '\n';
		it++;
	}
	
	ofs.flush();
	ofs.close();
}

void CMaildirStore::restoreUids()
{
	char aUid[70]; // The max uid length as specified by the POP3 RFC is 70B
	std::ifstream ifs;
	
	ifs.open(CFileSystem::MakeFullPath(m_path) + "uids");
	
	if (!ifs.good()) // File doesn't exit
		return;
	
	m_uids.clear();
	while (ifs.getline(aUid, 70))
		m_uids.push_back(CString(aUid));
		
	ifs.close();
}

//******************************************************************************
// Takes in a VMessageRefList and will remove any message with a UID of a
// message that has already been retrieved. If prune is true, the current UID
// list will be cleared and replaced by that of the messages already retrieved.
// Prune should be true only when called with aMsgs corresponding to all
// messages on the server.
//******************************************************************************
void CMaildirStore::filterAlreadyRetrievedMessages(VMessageRefList& aMsgs,
                                                   bool prune)
{
	std::vector<std::string> newUids; // For use when prune is true
	VMessageRefList::iterator it = aMsgs.begin();
	int startAt = 0;
	int i;
	bool firstLoop;
	bool deleted;
	
	while (it != aMsgs.end())
	{
		// firstLoop effectively ensures the loop runs because we want it to
		// stop at the second encounter of startAt, but if m_uids is empty, we
		// do not want to loop at all
		firstLoop = m_uids.size() > 0;
		deleted = false;
		
		// This loop is optimized for a message list and uid list stored in order
		for (i = startAt; i != startAt || firstLoop; (++i %= m_uids.size()))
		{
			firstLoop = false;
			
			if (vmime::utility::stringUtils::trim((*it)->getUniqueId()) == m_uids[i]) // Message exists on store
			{
				aMsgs.erase(it);
				if (prune)
					newUids.push_back(m_uids[i]);
					
				startAt = (++i % m_uids.size());
				deleted = true;
				break;
			}
		}
		
		if (!deleted)
			it++;
			
	}
	
	if (prune)
	{
		m_uids.clear();
		m_uids = newUids;
	}
}

int CMaildirStore::deleteMessage(const int num)
{
	std::vector<int> nums;
	nums.push_back(num);
	
	return deleteMessages(nums);
}

int CMaildirStore::deleteMessages(std::vector<int> nums)
{
	int toRet = 0;
	
	// The lock/unlock must happen outside the try/catch!!
	lock();
	
	try
	{
		connect();
		open(V_MODE_READ_WRITE);
		m_folder_ref->deleteMessages(nums);
	}
	catch (vmime::exception& e)
	{
		toRet = -1;
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		toRet = -1;
		S_EXCEPT(e);
	}
	
	try
	{
		close();
		disconnect();
	}
	catch (vmime::exception& e)
	{
		toRet = -1;
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		toRet = -1;
		S_EXCEPT(e);
	}
	
	// The lock/unlock must happen outside the try/catch!!
	unlock();
	
	return toRet;
}

CMessageHeader* CMaildirStore::getMessageHeader(VMessageRef aMsg)
{
	CMessageHeader* outputHeader = new CMessageHeader(m_mailbox);
	int msgFlags;
	vmime::ref<const vmime::header> msgHeader;
	vmime::charset ch(vmime::charsets::US_ASCII);
	
	try
	{
		msgFlags = aMsg->getFlags();
		msgHeader = aMsg->getHeader();
	}
	catch (vmime::exceptions::unfetched_object& e)
	{
		m_folder_ref->fetchMessage(aMsg, vmime::net::folder::FETCH_FULL_HEADER |
		                                 vmime::net::folder::FETCH_FLAGS);
		msgFlags = aMsg->getFlags();
		msgHeader = aMsg->getHeader();
	}
	
	if (msgHeader->hasField("To"))
		outputHeader->to = msgHeader->To()->getValue().dynamicCast<const vmime::addressList>();
	if (msgHeader->hasField("From"))
		outputHeader->from = msgHeader->From()->getValue().dynamicCast<const vmime::mailbox>();
	if (msgHeader->hasField("Cc"))
		outputHeader->cc = msgHeader->Cc()->getValue().dynamicCast<const vmime::addressList>();
	if (msgHeader->hasField("Bcc"))
		outputHeader->bcc = msgHeader->Bcc()->getValue().dynamicCast<const vmime::addressList>();
	if (msgHeader->hasField("Date"))
		outputHeader->date = convertTime(msgHeader->Date()->getValue().dynamicCast<const vmime::datetime>());
	if (msgHeader->hasField("Subject"))
		outputHeader->subject = msgHeader->Subject()->getValue().dynamicCast<const vmime::text>()->getConvertedText(ch);
		
	outputHeader->num = aMsg->getNumber();
	outputHeader->forwarded = (msgFlags & vmime::net::message::FLAG_PASSED) != 0;
	outputHeader->replied = (msgFlags & vmime::net::message::FLAG_REPLIED) != 0;
	outputHeader->unread = (msgFlags & vmime::net::message::FLAG_SEEN) == 0;
	outputHeader->marked = false;
	
	return outputHeader;
}

CString CMaildirStore::getBody(CMessageHeader* aMsg)
{
	vmime::string out;
	
	// The lock/unlock must happen outside the try/catch!!
	lock();
	
	try
	{
		connect();
		open(V_MODE_READ_ONLY);
		
		VMessageRef msg = m_folder_ref->getMessage(aMsg->num);
		VStructureRef msgStruct;
		
		try
		{
			msgStruct = msg->getStructure();
		}
		catch (vmime::exceptions::unfetched_object& e)
		{
			m_folder_ref->fetchMessage(msg, vmime::net::folder::FETCH_STRUCTURE);
			msgStruct = msg->getStructure();
		}
		
		if (msgStruct->getPartCount() > 0)
		{
			// Part 0 is assumed to be the body
			VPartRef msgPart = msgStruct->getPartAt(0);
			
			// This loop handles multipart messages and assumes the first part
			// is the text one
			while (msgPart->getPartCount() > 0)
				msgPart = msgPart->getPartAt(0);
			
			vmime::string str;
			vmime::utility::outputStreamStringAdapter oss(out);
			msg->extractPart(msgPart, oss);
		}
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
	}
	
	try
	{
		close();
		disconnect();
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
	}
	
	// The lock/unlock must happen outside the try/catch!!
	unlock();
	
	return CString(out);
}

void CMaildirStore::markRead(CMessageHeader* aMsg, bool aIsRead)
{
	// The lock/unlock must happen outside the try/catch!!
	lock();
	
	try
	{
		connect();
		open(V_MODE_READ_WRITE);
		m_folder_ref->setMessageFlags(aMsg->num, aMsg->num,
		                              vmime::net::message::FLAG_SEEN,
		                              aIsRead ? vmime::net::message::FLAG_MODE_ADD
		                                      : vmime::net::message::FLAG_MODE_REMOVE);
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
	}
	
	try
	{
		close();
		disconnect();
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
	}
	
	// The lock/unlock must happen outside the try/catch!!
	unlock();
}

void CMaildirStore::sortMessageList(CMailbox::TMessageList* aMsgList)
{
	std::sort(aMsgList->begin(), aMsgList->end(), messageHeaderBinaryPredicate);
}

time_t CMaildirStore::convertTime(vmime::ref<const vmime::datetime> aDatetime)
{
	vmime::datetime uTime(vmime::utility::datetimeUtils::toUniversalTime(*aDatetime));
	time_t t;
	struct tm utm;
	
	utm.tm_sec = uTime.getSecond();
	utm.tm_min = uTime.getMinute();
	utm.tm_hour = uTime.getHour();
	utm.tm_mday = uTime.getDay();
	utm.tm_mon = uTime.getMonth() - 1;
	utm.tm_year = uTime.getYear() - 1900;
	utm.tm_wday = uTime.getWeekDay();
	utm.tm_yday = -1;
	utm.tm_isdst = -1;
	
	t = mktime(&utm);

	return (t + (aDatetime->getZone() * 60));
}

/* Notifies the progress listener that a retrieval has started */
void CMaildirStore::start(const int predictedTotal)
{
	int err;
	
	SMailboxMsg* startMsg = new SMailboxMsg(EMailboxMsgType_StoringStarted);
	startMsg->count = predictedTotal;
	
	if ((err = sceKernelSendMbx(m_mailbox_mbx, startMsg)) < 0)
	{
		CLog::Error("sceKernelSendMbx returned %x\n", err);
	}
}

/* `current` has been retrieved */
void CMaildirStore::progress(const int current, const int currentTotal)
{
	int err;
	
	SMailboxMsg* progressMsg = new SMailboxMsg(EMailboxMsgType_StoredOne);
	progressMsg->index = current;
	progressMsg->count = currentTotal;
	
	if ((err = sceKernelSendMbx(m_mailbox_mbx, progressMsg)) < 0)
	{
		CLog::Error("sceKernelSendMbx returned %x\n", err);
	}
}

/* Retrieval done */
void CMaildirStore::stop(const int total)
{
	int err;
	
	SMailboxMsg* stopMsg;
	
	if (total == 0)
		stopMsg = new SMailboxMsg(EMailboxMsgType_StoredNone);
	else
		stopMsg = new SMailboxMsg(EMailboxMsgType_StoringDone);
	
	stopMsg->count = total;
	
	if ((err = sceKernelSendMbx(m_mailbox_mbx, stopMsg)) < 0)
	{
		CLog::Error("sceKernelSendMbx returned %x\n", err);
	}
}

void CMaildirStore::failed(CString aErrorText)
{
	int err;
	
	SMailboxMsg* failedMsg = new SMailboxMsg(EMailboxMsgType_StoreFailed);
	failedMsg->errorText = aErrorText;
	
	if ((err = sceKernelSendMbx(m_mailbox_mbx, failedMsg)) < 0)
	{
		CLog::Error("sceKernelSendMbx returned %x\n", err);
	}
}
