/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CSMTPSender.cpp

  Description :	

*******************************************************************************/

#include "CSMTPSender.h"
#include "CConnectionManager.h"
#include "CLog.h"
#include "CAssert.h"
#include "CMailbox.h"

#include <vmime/vmime.hpp>
#include <vmime/platforms/psp/pspHandler.hpp>

CSMTPSender::CSMTPSender(CAccount* aAccount, SceUID aMailboxMbx,
                               SceUID aSenderMbx, vmime::ref<CCertificateManager> aCertManager) :
	CMailSender(aMailboxMbx, aSenderMbx, aCertManager),
	m_account(*aAccount),
	m_mailbox_mbx(aMailboxMbx),
	m_sender_mbx(aSenderMbx),
	m_total(0),
	m_sent(0),
	m_listAwaitingCert(NULL)
{
	// Build the URL
	CString smtpUrl;
	if (m_account.outgoing_server.Substring(0, 7).IEquals("smtp://") || m_account.outgoing_server.Substring(0, 7).IEquals("smtps://"))
	{
		smtpUrl = m_account.outgoing_server;
	}
	else
	{
		if (m_account.outgoing_server_tls)
			smtpUrl = "smtps://";
		else
			smtpUrl = "smtp://";
			
		smtpUrl += m_account.outgoing_server;
		if (m_account.outgoing_server_port != 0)
		{
			smtpUrl += ":";
			smtpUrl += m_account.outgoing_server_port;
		}
	}
	
	try
	{
		vmime::platform::setHandler <vmime::platforms::psp::pspHandler>();

		// Create a new session
		vmime::utility::url url(smtpUrl);
		url.setPort(vmime::port_t(m_account.outgoing_server_port));
		m_sess_ref = vmime::create <vmime::net::session>();
		
		// Create an instance of the transport service
		m_tr_ref = m_sess_ref->getTransport(url);
		m_tr_ref->setCertificateVerifier(m_certManager);

		if (m_account.outgoing_server_username != "" || m_account.outgoing_server_password != "")
		{
			m_tr_ref->setProperty("options.need-authentication", true);
			m_tr_ref->setProperty("auth.username", m_account.outgoing_server_username);
			m_tr_ref->setProperty("auth.password", m_account.outgoing_server_password);
		}
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
	}
}

CSMTPSender::~CSMTPSender()
{
}


void CSMTPSender::connect()
{
	if (!m_tr_ref)
		throw vmime::exception("Transport was uninitialized");
		
	if (!m_tr_ref->isConnected())
		m_tr_ref->connect();
}

void CSMTPSender::disconnect()
{
	if (!m_tr_ref)
		throw vmime::exception("Transport was uninitialized.");
		
	if (m_tr_ref->isConnected())
		m_tr_ref->disconnect();
}

void CSMTPSender::send(CMailbox::TSendMsgList* msgList)
{
	m_sent = 0;
	m_pSentList = new CMailbox::TSendMsgList();

	if (msgList && msgList->size() != 0) {
		m_total = msgList->size();
	} else {
		failed(msgList, "No messages to send.");
		disconnected();
		return;
	}

	try
	{
		connect();
	
		while(!msgList->empty()) {
			// Get msg from list
			CSendMsg* msg = msgList->back();
			msgList->pop_back();

			// Build the message
			vmime::messageBuilder mb;
			mb.setSubject(vmime::text(msg->subject));
			mb.setExpeditor(vmime::mailbox(CConfigFile::s_pSettings->selectedAccount->email_address));
			mb.setRecipients(msg->to);
			mb.setBlindCopyRecipients(msg->bcc);
			mb.setCopyRecipients(msg->cc);
			mb.getTextPart()->setText(vmime::create<vmime::stringContentHandler>(msg->body));
			
			// Message construction
			vmime::ref<vmime::message> vmimeMsg = mb.construct();

			// Append a reply to field (no message builder method to do this
			vmime::headerFieldFactory* hfFactory = vmime::headerFieldFactory::getInstance();
			vmime::ref<vmime::header> hdr = vmimeMsg->getHeader();
			vmime::ref<vmime::headerField> replyToField =
				hfFactory->create(vmime::fields::REPLY_TO);
			replyToField->setValue(vmime::text(CConfigFile::s_pSettings->selectedAccount->reply_to));
			hdr->appendField(replyToField);

			// Send the message
			m_tr_ref->send(vmimeMsg);
			
			m_pSentList->push_back(msg);
			successfulMessageSend();
		}
	}
	catch (CCertificateException& e)
	{
		// In this case, we don't want to send a Disconnected message 'cause
		// it'd bring down the wifi connection. We also don't need to disconnect
		// since we never connected.
		if (m_listAwaitingCert == NULL || m_listAwaitingCert == msgList)
		{
			m_listAwaitingCert = msgList;
			m_certComm.sendCertRequest(e.cert());
			return;
		}
		else
		{
			// There was an existing list awaiting for a cert reply.
			const char* err = "A previous send is already waiting for a cert reply.";
			CLog::Error(err);
			failed(msgList, err);
		}
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
		failed(msgList, e.what());
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
		failed(msgList, e.what());
	}
	
	try
	{
		disconnect();
	}
	catch (vmime::exception& e)
	{
		V_EXCEPT(e);
	}
	catch (std::exception& e)
	{
		S_EXCEPT(e);
	}
	
	disconnected(); // Sends a Disconnected message
}

void CSMTPSender::certDeclined()
{
	failed(m_listAwaitingCert, "Certificate declined, message(s) not sent.");
	disconnected();
	// Don't delete the list, the mailbox is responsible for that!
	m_listAwaitingCert = NULL;
}

void CSMTPSender::certAccepted()
{
	// Don't delete the list, the mailbox is responsible for that! Also, we need
	// to clear m_listAwaitingCert before calling send because if a cert request
	// happens again, we don't want to clear m_listAwaitingCert blindly
	// afterwards.
	CMailbox::TSendMsgList* aListToSend = m_listAwaitingCert;
	m_listAwaitingCert = NULL;
	send(aListToSend);
}

void CSMTPSender::successfulMessageSend()
{
	int err;
	SMailboxMsg* mbxMsg;

	m_sent++;

	if (m_sent != m_total) {
		mbxMsg = new SMailboxMsg(EMailboxMsgType_SentOne);
		mbxMsg->index = m_sent;
		mbxMsg->count = m_total;
	} else {
		mbxMsg = new SMailboxMsg(EMailboxMsgType_SendDone);
		mbxMsg->count = m_total;
		mbxMsg->sentList = m_pSentList;
		m_pSentList = NULL;
	}

	if ((err = sceKernelSendMbx(m_mailbox_mbx, mbxMsg)) < 0)
	{
		CLog::Error("sceKernelSendMbx returned %x\n", err);
	}
}

void CSMTPSender::failed(CMailbox::TSendMsgList* msgList, CString aErrorText)
{
	int err;
	
	SMailboxMsg* failedMsg = new SMailboxMsg(EMailboxMsgType_SendFailed);
	
	failedMsg->sentList = m_pSentList;
	failedMsg->errorText = aErrorText;
	m_pSentList = NULL;
	failedMsg->failedList = msgList;

	if ((err = sceKernelSendMbx(m_mailbox_mbx, failedMsg)) < 0)
	{
		CLog::Error("sceKernelSendMbx returned %x\n", err);
	}
}

void CSMTPSender::disconnected()
{
	int err;
	
	SMailboxMsg* disconnectedMsg = new SMailboxMsg(EMailboxMsgType_SendDisconnected);
	
	if ((err = sceKernelSendMbx(m_mailbox_mbx, disconnectedMsg)) < 0)
	{
		CLog::Error("sceKernelSendMbx returned %x\n", err);
	}
}


#if 0

/* Progress listener functions follow */

/* When true is returned, will cancel the retrieval; not supported by POP3 */
const bool CSMTPSender::cancel() const
{
	return false;
}

/* Notifies the progress listener that a retrieval has started */
void CSMTPSender::start(const int predictedTotal)
{
	int err;
	
	SMailboxMsg* startMsg = new SMailboxMsg;
	startMsg->m_type = EMailboxMsgType_RetrievalStarted;
	startMsg->count = predictedTotal;
	
	if ((err = sceKernelSendMbx(m_mailbox_mbx, startMsg)) < 0)
	{
		CLog::Error("sceKernelSendMbx returned %x\n", err);
	}
}

/* `current` has been retrieved */
void CSMTPSender::progress(const int current, const int currentTotal)
{
	int err;
	
	SMailboxMsg* progressMsg = new SMailboxMsg;
	progressMsg->m_type = EMailboxMsgType_RetrievedOne;
	progressMsg->index = current;
	progressMsg->count = currentTotal;
	
	if ((err = sceKernelSendMbx(m_mailbox_mbx, progressMsg)) < 0)
	{
		CLog::Error("sceKernelSendMbx returned %x\n", err);
	}
}

/* Retrieval done */
void CSMTPSender::stop(const int total)
{
	int err;
	
	SMailboxMsg* stopMsg = new SMailboxMsg;
	
	if (total == 0)
		stopMsg->m_type = EMailboxMsgType_RetrievedNone;
	else
		stopMsg->m_type = EMailboxMsgType_RetrievalDone;
	
	stopMsg->count = total;
	
	if ((err = sceKernelSendMbx(m_mailbox_mbx, stopMsg)) < 0)
	{
		CLog::Error("sceKernelSendMbx returned %x\n", err);
	}
}
#endif


