/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	CSkinManager.cpp

  Description :	

  Last Modified $Date: 2007/01/03 05:31:14 $

  $Revision: 1.3 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include "CSkinManager.h"
#include "CTextureManager.h"
#include "CFont.h"
#include "CFrameWork.h"
#include "CConfigFile.h"
#include "tinyxml.h"
#include "CGfx.h"
#include "CFileSystem.h"
#include "CCachedTexture.h"

//******************************************************************************
//   Local Macros
//******************************************************************************

//******************************************************************************
//   Local Constants
//******************************************************************************

//******************************************************************************
//   Static Prototypes
//******************************************************************************

//******************************************************************************
//   Global Variables
//******************************************************************************

//******************************************************************************
//   Static Variables
//******************************************************************************
bool			CSkinManager::s_bConfigLoaded( false );
TiXmlDocument *		CSkinManager::s_pSkinConfigFile( NULL );
const CString		CSkinManager::s_szSkinFolder( "Data/Skins/" );
CSkinTexture *		CSkinManager::s_pComponents[ CSkinManager::MAX_SKIN_COMPONENTS ];
const CString		CSkinManager::s_szComponentNames[ MAX_SKIN_COMPONENTS ] =
{
	"font.png",					// SC_FONT,
	"background.png",			// SC_BACKGROUND,
	
	"icon_messages.png",		// SC_ICON_MESSAGES,
	"icon_compose.png",			// SC_ICON_COMPOSE
	"icon_contacts.png",		// SC_ICON_CONTACTS
	"icon_settings.png",		// SC_ICON_SETTINGS
	"icon_game.png",			// SC_ICON_GAME
	"icon_exit.png",			// SC_ICON_EXIT
	"icon_newmail.png",			// SC_ICON_NEW_MAIL
	
	"icon_mail_open.png",		// SC_ICON_MAIL_OPEN,
	"icon_mail_forward.png",	// SC_ICON_MAIL_FORWARD,
	"icon_mail_reply.png",		// SC_ICON_MAIL_REPLY,
	"icon_mail_replyall.png",	// SC_ICON_MAIL_REPLYALL,
	"icon_mail_mark.png",		// SC_ICON_MAIL_MARK,

	"button_cross.png",			// SC_BUTTON_CROSS,
	"button_circle.png",		// SC_BUTTON_CIRCLE,
	"button_square.png",		// SC_BUTTON_SQUARE,
	"button_triangle.png",		// SC_BUTTON_TRIANGLE,
	"button_ltrigger.png",		// SC_BUTTON_LTRIGGER,
	"button_rtrigger.png",		// SC_BUTTON_RTRIGGER,
	"button_start.png",			// SC_BUTTON_START,
	"button_select.png",		// SC_BUTTON_SELECT,
	"button_up.png",			// SC_BUTTON_UP,
	"button_down.png",			// SC_BUTTON_DOWN,
	"button_right.png",			// SC_BUTTON_RIGHT,
	"button_left.png",			// SC_BUTTON_LEFT,

	"battery_00.png",			// SC_BATTERY_00,
	"battery_01.png",			// SC_BATTERY_01,
	"battery_02.png",			// SC_BATTERY_02,
	"battery_03.png"			// SC_BATTERY_03,

};

//******************************************************************************
//   Class Definition
//******************************************************************************

//******************************************************************************
//******************************************************************************
//
//	Skin texture class implementation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
CSkinTexture::CSkinTexture()
:	m_pTexture( NULL )
{
}

//******************************************************************************
//	
//******************************************************************************
CSkinTexture::~CSkinTexture()
{
	SAFE_DELETE( m_pTexture );
}

//******************************************************************************
//	
//******************************************************************************
void	CSkinTexture::Load( const CString & texture_name )
{
	SAFE_DELETE( m_pTexture );

	m_pTexture = CTextureManager::CreateCached( texture_name );
}

//******************************************************************************
//	
//******************************************************************************
CTexture *	CSkinTexture::GetTexture() const
{
	return m_pTexture;
}


//******************************************************************************
//******************************************************************************
//
//	Skin manager class implementation
//
//******************************************************************************
//******************************************************************************

//******************************************************************************
//	
//******************************************************************************
void	CSkinManager::Open()
{
	for ( u32 i = 0; i < MAX_SKIN_COMPONENTS; ++i )
	{
		s_pComponents[ i ] = new CSkinTexture();
	}

	SetCurrentSkin( CConfigFile::s_pSettings->skin  );
}

//******************************************************************************
//	
//******************************************************************************
void	CSkinManager::Close()
{
	for ( u32 i = 0; i < MAX_SKIN_COMPONENTS; ++i )
	{
		SAFE_DELETE( s_pComponents[ i ] );
	}

	SAFE_DELETE( s_pSkinConfigFile );
	CGfx::FreeAllVRAM();
}

//******************************************************************************
//	
//******************************************************************************
void	CSkinManager::SetCurrentSkin( const CString & skin_name )
{
	CConfigFile::s_pSettings->skin = skin_name;

	for ( u32 i = 0; i < MAX_SKIN_COMPONENTS; ++i )
	{
		const CString	component( s_szSkinFolder + skin_name + "/" + s_szComponentNames[ i ] );

		s_pComponents[ i ]->Load( component );
	}

	/*if ( s_pComponents[ SC_FONT ]->GetTexture() != NULL )
	{
		s_pComponents[ SC_FONT ]->GetTexture()->Upload();
	}*/

	LoadConfig();

	OptimizeTextures();
}

//******************************************************************************
//	
//******************************************************************************
CSkinTexture *	CSkinManager::GetComponent( eSkinComponent component )
{
	return s_pComponents[ component ];
}

//******************************************************************************
//	
//******************************************************************************
void	CSkinManager::LoadConfig()
{
	SAFE_DELETE( s_pSkinConfigFile );

	s_pSkinConfigFile = new TiXmlDocument();

	ASSERT( s_pSkinConfigFile != NULL, "Failed to create the XML document" );

	if ( s_pSkinConfigFile != NULL )
	{
		const CString	config_file( s_szSkinFolder + CConfigFile::s_pSettings->skin + "/Skin.xml" );

		if ( s_pSkinConfigFile->LoadFile( config_file ) == true )
		{
			s_bConfigLoaded = true;
		}
		else
		{
			s_bConfigLoaded = false;
			TRACE( "%s\n", s_pSkinConfigFile->ErrorDesc() );
			ASSERT( 0, "Failed to load skin config file!" );
		}
	}
}

//******************************************************************************
//	
//******************************************************************************
V2	CSkinManager::GetV2( const CString & section, const CString & sub_section, const V2 & default_val )
{
	if ( s_bConfigLoaded == true )
	{
		TiXmlHandle		doc_handle( s_pSkinConfigFile );
		TiXmlElement *	p_xy( doc_handle.FirstChild( "skin" ).FirstChild( section ).FirstChild( sub_section ).Element() );

		if ( p_xy != NULL )
		{
			V2	val( default_val );

			if ( p_xy->Attribute( "x" ) != NULL )	val.x = atof( p_xy->Attribute( "x" ) );
			if ( p_xy->Attribute( "y" ) != NULL )	val.y = atof( p_xy->Attribute( "y" ) );

			return val;
		}
	}

	return default_val;
}

//******************************************************************************
//	
//******************************************************************************
ARGB	CSkinManager::GetColor( const CString & section, const CString & sub_section, const ARGB & default_val )
{
	if ( s_bConfigLoaded == true )
	{
		TiXmlHandle		doc_handle( s_pSkinConfigFile );
		TiXmlElement *	p_color( doc_handle.FirstChild( "skin" ).FirstChild( section ).FirstChild( sub_section ).Element() );

		if ( p_color != NULL )
		{
			ARGB	color( default_val );

			if ( p_color->Attribute( "a" ) != NULL )	color.a = atoi( p_color->Attribute( "a" ) );
			if ( p_color->Attribute( "r" ) != NULL )	color.r = atoi( p_color->Attribute( "r" ) );
			if ( p_color->Attribute( "g" ) != NULL )	color.g = atoi( p_color->Attribute( "g" ) );
			if ( p_color->Attribute( "b" ) != NULL )	color.b = atoi( p_color->Attribute( "b" ) );

			return color;
		}
	}

	return default_val;
}

//******************************************************************************
//	
//******************************************************************************
float	CSkinManager::GetFloat( const CString & section, const CString & sub_section, const float & default_val )
{
	if ( s_bConfigLoaded == true )
	{
		TiXmlHandle		doc_handle( s_pSkinConfigFile );
		TiXmlElement *	p_val( doc_handle.FirstChild( "skin" ).FirstChild( section ).FirstChild( sub_section ).Element() );

		if ( p_val != NULL )
		{
			float	val( default_val );

			if ( p_val->Attribute( "val" ) != NULL )	val = atof( p_val->Attribute( "val" ) );

			return val;
		}
	}

	return default_val;
}

//******************************************************************************
//	
//******************************************************************************
void	CSkinManager::OptimizeTextures()
{
	CGfx::FreeAllVRAM();

	for (int i = 0; i < MAX_SKIN_COMPONENTS; i++)
	{
		if (s_pComponents[i]->GetTexture() != NULL)
			s_pComponents[i]->GetTexture()->Optimize();
	}
}

//******************************************************************************
//	
//******************************************************************************
void	CSkinManager::UploadFontTexture()
{
	if ( s_pComponents[ SC_FONT ]->GetTexture() != NULL )
	{
		s_pComponents[ SC_FONT ]->GetTexture()->Upload();
	}
}

//******************************************************************************
//	
//******************************************************************************
CString CSkinManager::GetUiFontPath()
{
	CString fontPath = s_szSkinFolder;
	fontPath += CConfigFile::s_pSettings->skin;
	fontPath += "/ui.ttf";
	return CFileSystem::MakeFullPath(fontPath);
}

//******************************************************************************
//	
//******************************************************************************
CString CSkinManager::GetTextFontPath()
{
	CString fontPath = s_szSkinFolder;
	fontPath += CConfigFile::s_pSettings->skin;
	fontPath += "/text.ttf";
	return CFileSystem::MakeFullPath(fontPath);
}

//*******************************  END OF FILE  ********************************
