/*******************************************************************************

  PSPoste - Wireless Email Client for the Sony PSP
  Copyright (C) Benjamin Biron, Sajeeth Cherian, David Cummings

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  Module :	main_fw15.c

  Description :	Main file for release mode targetting the 1.5 firmware.

  Last Modified $Date: 2007/02/08 04:17:16 $

  $Revision: 1.3 $

*******************************************************************************/

//******************************************************************************
//   Include Files
//******************************************************************************
#include <pspnet.h>
#include <pspthreadman.h>
#include "main_globals.h"
#include "CFrameWork.h"
#include "CConnectionManager.h"
#include "CLog.h"

PSP_MODULE_INFO(PSPoste, 0x1000, 0, 1 ); //0->1 ?
PSP_MAIN_THREAD_ATTR( 0 );

PSP_HEAP_SIZE_KB(PSPOSTE_HEAP_SIZE_MB * 1024);

//******************************************************************************
//   Type Definitions
//******************************************************************************
// List of modules to load - type is 0 if the module belongs to the kernel, 1 if
// it's a user module.
struct module_list {
    int         type;
    const char *path;
} module_list[] = {
	{ 0, "flash0:/kd/ifhandle.prx" },
	{ 1, "flash0:/kd/pspnet.prx" },
	{ 1, "flash0:/kd/pspnet_inet.prx" },
	{ 1, "flash0:/kd/pspnet_apctl.prx" },
	{ 1, "flash0:/kd/pspnet_resolver.prx"}
};

//******************************************************************************
//   Local Constants
//******************************************************************************
const int module_count = sizeof(module_list) / sizeof(module_list[0]);

// Exceptions in release mode currently throw a bluescreen of debug info and
// regs, this screen disapears rather quickly if the exception occurs in the
// thread running user_main (ie: most of the application) since the thread exits
// and the kernel bootstrap thread in turn terminates. Uncomment the line to
// ensure the bluescreen stays up even if the exception occurs in the user_main
// thread. Note that a hard reboot is then required!
//#define SLEEP_ON_EXCEPTION

//******************************************************************************
// This function is taken from the net samples in PSPSDK
//******************************************************************************
SceUID load_module(const char *path, int flags, int type)
{
	SceKernelLMOption option;
	SceUID mpid;

	/* If the type is 0, then load the module in the kernel partition, otherwise load it
	   in the user partition. */
	if (type == 0)
		mpid = 1;
	else
		mpid = 2;

	memset(&option, 0, sizeof(option));
	option.size = sizeof(option);
	option.mpidtext = mpid;
	option.mpiddata = mpid;
	option.position = 0;
	option.access = 1;

	return sceKernelLoadModule(path, flags, &option);
}

void storeRegistersToFile(PspDebugRegBlock *regs)
{
	int i;
	FILE* f = fopen("crash.log", "w");
	
	// Dump the frames
	fprintf(f, "--- Frames ---\n");
	for (i = 0; i < 6; i++)
	{
		fprintf(f, "frame[%d]: 0x%x\n", i, regs->frame[i]);
	}
	
	// Dump the GPRs
	fprintf(f, "\n--- GPRs ---\n");
	for (i = 0; i < 32; i++)
	{
		fprintf(f, "r[%d]: 0x%x\n", i, regs->r[i]);
	}
	
	fprintf(f, "\nstatus: 0x%x\n", regs->status);
	fprintf(f, "lo: 0x%x\n", regs->lo);
	fprintf(f, "hi: 0x%x\n", regs->hi);
	fprintf(f, "badvaddr: 0x%x\n", regs->badvaddr);
	fprintf(f, "cause: 0x%x\n", regs->cause);
	fprintf(f, "epc: 0x%x\n", regs->epc);
	
	// Dump the FPRs
	fprintf(f, "\n--- FPRs ---\n");
	for (i = 0; i < 32; i++)
	{
		fprintf(f, "fpr[%d]: %f\n", i, regs->fpr[i]);
	}
	
	fclose(f);
}

//******************************************************************************
// The exception handler will print a bluescreen of regs and close the logs
//******************************************************************************
// Note that the simple presence of this function when in _PSPLINK (user mode),
// causes the app to not launch! Also note that psplink has its own exception
// handler to print out the exception to the console.
void exception_handler(PspDebugRegBlock *regs)
{
	pspDebugScreenInit();
	pspDebugScreenSetBackColor(0x00FF0000);
	pspDebugScreenSetTextColor(0xFFFFFFFF);
	pspDebugScreenClear();
	pspDebugScreenPrintf("Exception Details:\n");
	pspDebugDumpException(regs);
	storeRegistersToFile(regs);
	CLog::Close();
#ifdef SLEEP_ON_EXCEPTION
	sceKernelSleepThread();
#endif
}

//******************************************************************************
// Calling CLog::Message doesn't behave properly, so do nothing
//******************************************************************************
int stdout_handler(const char* data, int len)
{
	//CLog::Message("\n<stdout>\n%s\n</stdout>\n", data);
	return len;
}

//******************************************************************************
// Calling CLog::Error doesn't behave properly, so do nothing
//******************************************************************************
int stderr_handler(const char* data, int len)
{
	//CLog::Error("\n<stderr>\n%s\n</stderr>\n", data);
	return len;
}

//******************************************************************************
// This main runs in kernel mode and spawns a user mode thread that will run the
// PSPoste code
//******************************************************************************
int main(int argc, char* argv[])
{
	int	(*user_func)(SceSize argc, void* argp) = user_main; // Function pointer
	SceKernelThreadInfo thinfo; // Used to get the user thread exit status
	thinfo.size = sizeof(SceKernelThreadInfo);
	SceUID thid; // Thread id of user thread
	SceUID connManThid; // Thread id of connection manager
	
	// Initialise the debug output so we can catch any early DebugScreenPrintfs
	pspDebugScreenInit();

	// Open the log file now to catch any early log messages
	CLog::Open();
	
	// Install stdout and stderr handlers
	pspDebugInstallStdoutHandler(stdout_handler);
	pspDebugInstallStderrHandler(stderr_handler);
	
	// We drop the MSB of the function address to ensure user mode addressing
	user_func = (int (*)(SceSize, void*))((u32)user_func & 0x7fffffff);
	
	// NULLify the device and plain module checks, install the blue screen
	// exception handler and call the FA setKernelPatch
	pspSdkInstallNoDeviceCheckPatch();
	pspSdkInstallNoPlainModuleCheckPatch();
	pspDebugInstallErrorHandler(exception_handler);
	setKernelPatch();
	
	// Load the network drivers... this MUST happen in kernel mode
	if (pspSdkLoadInetModules() < 0)
	{
		ASSERT(0, "Error loading network drivers");
	}
	
	// Start the connection manager thread
	connManThid = sceKernelCreateThread("Conn Man Thread",
	                                    CConnectionManager::ConnManThreadCB,
	                                    0x11, CONN_MAN_THREAD_STACK_SIZE,
	                                    PSP_THREAD_ATTR_USER, NULL);

	if (connManThid & 0x80000000)
	{
		CLog::Error("Failed to create the connection manager thread: 0x%x\n", connManThid);
	}
	else
	{
		sceKernelStartThread(connManThid, 0, NULL);
	}
	
	// This path is used to set the working directory in the user thread, it's
	// also used in this function to reboot the app
	strncpy((char*)&s_ebootPath, argv[0], 255);
	
	// 0x11 is the priority, 0x40000 is the stack size and corresponds to the
	// main thread stack size as determined from the crt0.c _start function
	// Note that 0x2D000 is the max free memory with PSPLink running
	// Size was 0x2D000
	thid = sceKernelCreateThread("User Mode Thread", user_func, 0x11,
	                             USER_THREAD_STACK_SIZE, PSP_THREAD_ATTR_USER,
	                             NULL);

	if (thid & 0x80000000)
	{
		CLog::Error("Failed to create the user thread: 0x%x\n", thid);
	}
	else
	{
		// Start thread with no parameters
		sceKernelStartThread(thid, 0, NULL);
	}
	
	// Wait for the user thread to terminate
	sceKernelWaitThreadEnd(thid, NULL);
	
	// Kill the conn man then clean up
	CConnectionManager::RequestKill();
	sceKernelWaitThreadEnd(connManThid, NULL);
	
	// Now that the user thread has resumed, this thread may resume, let's check
	// the user thread return status
	sceKernelReferThreadStatus(thid, &thinfo);
	
	// If the user thread exited with the REBOOT_RETURN_CODE, let's reboot the
	// app. Else, exit the game. Note that to prevent exiting the game on
	// failure (to look at the bluescreen for instance), should check for exit
	// code != 0, in that case just call sceKernelSleepThread
	if (thinfo.exitStatus == REBOOT_RETURN_CODE)
		CFrameWork::RunExecutable(s_ebootPath);
	else
		sceKernelExitGame();
	
	return 0;
}
